function [ pts3d, pts2d ] = add_outliers( pts3d, pts2d, frac, xrange, yrange, zrange)
%duplicate random 2d/3d points with incorrect correspondences to produce
%outliers
%frac defines the desired fraction of outlier contamination (outliers/total)

%how many outliers do we need to fulfill outlier_frac?
%frac=out/(in+out)
%out = in*frac/(1-frac)
n_in=size(pts3d,2);
n_outliers = round(n_in*frac/(1-frac));

%randomly choose points for duplication
outlier2d = randi(n_in,n_outliers,1);
outlier3d = randi(n_in,n_outliers,1);

%don't insert duplicate inliers
accidental_inlier = outlier2d==outlier3d;
max_index = outlier3d==n_in;
increment = accidental_inlier .*(-max_index); %add one to all accidental inliers, but remove one if they are already max
outlier3d = outlier3d+increment;

%add a small amount of noise to prevent too much degeneracy (this
%degeneracy undermines the outliers because most combinations can't be used in ransac for example)
%noise is according to range of points, so configuration is unchanged
outlier_ptd3d = pts3d(:,outlier3d)+[xrand(1,n_outliers,xrange*1e-2);...
                                 xrand(1,n_outliers,yrange*1e-2);...
                                 xrand(1,n_outliers,zrange*1e-2)];
pts3d = [pts3d outlier_ptd3d];
pts2d = [pts2d pts2d(:,outlier2d)];


end

