%takes a camera matrix P and decomposes it into 
%
% C is camera centre in world co-ords R is rotation from world to camera co-ords 

function [K,R,C] = decomposeP(P)
	K = zeros(3);
	
	M = P(:,1:3);
	m = P(:,4);
	m3norm = sqrt(sum(sum(M(3,:).^2)));
	
	M = sign(det(M)).*M./m3norm;
	
	K(3,3) = 1;
	K(2,3) = M(2,:)*M(3,:)';
	K(2,2) = sqrt(M(2,:)*M(2,:)' - K(2,3)^2);
	K(1,3) = M(1,:)*M(3,:)';
	K(1,2) = (M(1,:)*M(2,:)' - K(1,3)*K(2,3))/K(2,2);
	K(1,1) = sqrt(M(1,:)*M(1,:)' - K(1,2)^2 - K(1,3)^2);
	
	R = (K\M)';
	
	C = null(P);
	C = C(1:3)./C(4);
	
	if sum(sum(abs(M - K*(R'))) > 1e-10) > 0 || sum(abs(P*[C;1])>1e-6)>0
		fprintf(2,'ERROR IN P decomposition!!!\n');
	end
	
end

