%% prepare the data - mostly adapted from the synthetic benchmarks
tic
rng(1234);

npt = 100;
focal= 800;
Xc= [xrand(1,npt,[-2 2]); xrand(1,npt,[-2 2]); xrand(1,npt,[4 8])];

% projection
xx= [Xc(1,:)./Xc(3,:); Xc(2,:)./Xc(3,:)]*focal;
[ Xc, xx] = add_outliers( Xc, xx, 0.10, [-2 2], [-2 2], [4 8]);

t_GT = mean(Xc,2);
R_GT = eye(3);
Xw= R_GT \ bsxfun(@minus, Xc, t_GT);

xx = xx + randn('like', xx) * focal/10; % add 10% gaussian noise 

ROT_TYPE_RV = 0;
ROT_TYPE_EA = 1;
ROT_TYPE_Q = 2;

ROT_TYPE_HAR= -2;

%% prepare the rotations
rstep1=0.003125;
rstep2=0.003125;
r1 = -1:rstep1:1;
r2 = -1:rstep2:1;
r3 = 0;

[r1, r2, r3] = meshgrid(r1, r2, r3);
% r1 = r1(:)';
% r2 = r2(:)';
% r3 = r3(:)';

%% error computation - ang

err_unrefined = zeros('like', r1);

for par = 1:numel(r1)
	R = ang2orth([r1(par) r2(par) r3(par)]);
	X2 = bsxfun(@plus, R * Xw, t_GT);
	xx2 = [X2(1,:)./X2(3,:); X2(2,:)./X2(3,:)]*focal;
	err_unrefined(par) = norm(xx - xx2, 'fro');
end

%% where do we converge? RV
cam.K = eye(3);
reps_to_use={ROT_TYPE_RV, ROT_TYPE_EA, ROT_TYPE_Q, ROT_TYPE_HAR};
names={'RV','EA','Q','HAR'};
num_techs=numel(reps_to_use);
fd=true;
rotsRV = zeros(3, numel(r1), num_techs);

err_refined = zeros(num_techs, numel(r1));
for technique_to_use=1:num_techs
    for par = 1:numel(r1)
        cam.R = ang2orth([r1(par) r2(par) r3(par)]) * R_GT;
        cam.C = -cam.R*t_GT;
        
        rep=reps_to_use{technique_to_use};
        [R, c] = pnp_update(t_GT, cam.R', Xw, xx/focal, [1 0 -inf fd], rep);

        rotsRV(:,par,technique_to_use) = orth2ang(R / R_GT);
        X2 = bsxfun(@plus, R * Xw, c);
        xx2 = [X2(1,:)./X2(3,:); X2(2,:)./X2(3,:)]*focal;
        err_refined(technique_to_use,par) = norm(xx - xx2, 'fro');        
    end
end

%include unrefined
err_refined(end+1,:) = err_unrefined;
num_techs=num_techs+1;
rotsRV(:,:,end+1) = nan;
names{end+1}='Initial';

for technique_to_use=1:num_techs    
    figure(200+technique_to_use);
    %heat map of reproj error
    hold off
    imagesc(reshape(log(err_refined(technique_to_use,:)), size(r1)))
    axis xy
    hold on
    
    %and draw global minima
    scatter(size(r1,1)/2, size(r1,2)/2, 1200, 'w.');    
    %find local minima
    errs = reshape(err_refined(technique_to_use,:),size(r1));
    neighbourhood = create_neighbourhood_stack( errs, 8);
    [min_errs, min_idxs] = min(neighbourhood, [], 3); %centre is 5th element of patch stack
    local_minima = min_idxs==5;
    %draw them on
    [minima_i, minima_j] = find(local_minima);
    scatter(minima_j, minima_i, 600/8, 'rd','filled');
    
    nrm=size(r1,1)/2-0.5;
    bs=1;
    tick_step=64;
    ax=gca;
    ax.XTick = (r1(1,1:tick_step:end)'+bs)*nrm+1;
    ax.XTickLabel = linspace(-1,1,size(r1,1)/tick_step+1);
    ax.YTick = (r1(1,1:tick_step:end)'+bs)*nrm+1;
    ax.YTickLabel = linspace(-1,1,size(r2,2)/tick_step+1);
    axis equal
    xlabel('Euler Angle 1 (rad.)'); ylabel('Euler Angle 2 (rad.)');
    xlim([1 size(r1,2)])
    ylim([1 size(r1,1)])
%     title(names{technique_to_use});
    mkdir('results_me/intuition');
    print('-depsc',['results_me/intuition/' names{technique_to_use} '.eps']);
    print('-dpng',['results_me/intuition/' names{technique_to_use} '.png']);
end
toc
