function euler_vector = orth2ang(rot_matrix) 
%extract euler angles from matrix. The returned angles are in the order: 
%Roll, Pitch, Yaw (rotation around Z, rotation around X, rotation around Y)

%read values from rot matrix into variables, relating to maths in logbook
a=-rot_matrix(2,3);
b=rot_matrix(1,1);
c=rot_matrix(1,2);
d=rot_matrix(3,1);
e=rot_matrix(3,2);
angle_sum=asin((c+d)/(a-1.0));
angle_diff=acos((b+e)/(a+1.0));
euler_vector=zeros(3,1);

%create first estimate of euler angles
euler_vector(2)=asin(a); %pitch

if(is_eq(sin(euler_vector(2)),1.0)) %angle sum ambiguous
    %lets set yaw to 0
    euler_vector(3)=0;
    %calc roll from angle difference
    euler_vector(1)=-angle_diff;
elseif(is_eq(sin(euler_vector(2)),-1.0)) %angle difference ambiguous
    %lets set yaw to 0
    euler_vector(3)=0;
    %calc roll from angle difference
    euler_vector(1)=angle_sum;
else
    euler_vector(3)=asin(rot_matrix(1,3)/cos(euler_vector(2))); %yaw
    euler_vector(1)=asin(rot_matrix(2,1)/cos(euler_vector(2))); %roll
end

%force them to be in the interval 0 to 2 pi
for i=1:3
    while(euler_vector(i)<0)
        euler_vector(i)=euler_vector(i)+(2.0*pi);
    end
    while(euler_vector(i)>(2.0*pi))
        euler_vector(i)=euler_vector(i)-(2.0*pi);
    end
end

%-------------optional Check for euler angles
repeat=true;
while(repeat)
    %create rot mat from current euler estimates
    euler_rotation_matrix = ang2orth(euler_vector);
    repeat=false;
    	%if the distance to negative value is less than the normal distance
	if(~is_eq(rot_matrix(1,3),0.0) && ~is_eq(euler_rotation_matrix(1,3),0.0) && abs(euler_rotation_matrix(1,3)+rot_matrix(1,3)) ...
			< abs(euler_rotation_matrix(1,3)-rot_matrix(1,3))) %value 0,2 in matrix checks if pitch is wrong
		if(euler_vector(2)<pi)		
			euler_vector(2)=pi-euler_vector(2);
		else
		
			euler_vector(2)=3.0*pi-euler_vector(2);
		end
		repeat=true;
		euler_rotation_matrix = ang2orth(euler_vector);
	end
	if(~is_eq(rot_matrix(3,3),0.0) && ~is_eq(euler_rotation_matrix(3,3),0.0) && abs(euler_rotation_matrix(3,3)+rot_matrix(3,3)) ...
			< abs(euler_rotation_matrix(3,3)-rot_matrix(3,3))) %value 2,2 in matrix checks if yaw is wrong
	
		if(euler_vector(3)<pi)		
			euler_vector(3)=pi-euler_vector(3);
		else
		
			euler_vector(3)=3.0*pi-euler_vector(3);
		end
		repeat=true;
		euler_rotation_matrix = ang2orth(euler_vector);
	end
	if(~is_eq(rot_matrix(2,2),0.0) && ~is_eq(euler_rotation_matrix(2,2),0.0) && abs(euler_rotation_matrix(2,2)+rot_matrix(2,2)) ...
			< abs(euler_rotation_matrix(2,2)-rot_matrix(2,2))) %value 1,1 in matrix checks if roll is wrong
		if(euler_vector(1)<pi)		
			euler_vector(1)=pi-euler_vector(1);
		else
		
			euler_vector(1)=3.0*pi-euler_vector(1);
		end
		repeat=true;
	end
end



end