(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[    272303,       7120]
NotebookOptionsPosition[    271048,       7080]
NotebookOutlinePosition[    271998,       7111]
CellTagsIndexPosition[    271955,       7108]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Rotation conversion hessians", "Subsubsection",
 CellChangeTimes->{{3.4529660967377443`*^9, 3.452966119824486*^9}, {
  3.6401072856196375`*^9, 3.640107291842189*^9}, {3.7207914603637543`*^9, 
  3.720791463945754*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", 
   RowBox[{
   "Define", " ", "the", " ", "different", " ", "representations", " ", "for",
     " ", "rotations"}], "*)"}], "\[IndentingNewLine]", 
  RowBox[{"(*", "Quaternion", "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"quat", "=", 
     RowBox[{"{", 
      RowBox[{"qx", ",", "qy", ",", " ", "qz", ",", " ", "qw"}], "}"}]}], 
    ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"qx", "*", "qx"}], " ", "+", " ", 
      RowBox[{"qy", "*", "qy"}], " ", "+", " ", 
      RowBox[{"qz", "*", "qz"}]}], "==", "1"}], ";"}], "\[IndentingNewLine]", 
   "\[IndentingNewLine]", 
   RowBox[{"(*", 
    RowBox[{"Rotation", " ", "vector"}], "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"rotVec", "=", 
     RowBox[{"{", 
      RowBox[{"rv1", ",", "rv2", ",", "rv3"}], "}"}]}], ";"}], 
   "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{"(*", 
    RowBox[{"Angle", " ", "axis"}], "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"angleAxis", "=", 
     RowBox[{"{", 
      RowBox[{"aa1", ",", " ", "aa2", ",", " ", "aa3", ",", " ", "aa4"}], 
      "}"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"aa1", "*", "aa1"}], " ", "+", " ", 
      RowBox[{"aa2", "*", "aa2"}], " ", "+", " ", 
      RowBox[{"aa3", "*", "aa3"}]}], "==", "1"}], ";"}], 
   "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{"(*", 
    RowBox[{"rotation", " ", "matrix"}], "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"R", " ", "=", " ", 
     RowBox[{
      RowBox[{"Array", "[", 
       RowBox[{"rotMat", ",", 
        RowBox[{"{", 
         RowBox[{"3", ",", "3"}], "}"}]}], "]"}], "//", "MatrixForm"}]}], 
    ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"R", ".", 
      RowBox[{"Transpose", "[", "R", "]"}]}], "\[Equal]", 
     RowBox[{"IdentityMatrix", "[", "3", "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"Det", "[", "R", "]"}], "\[Equal]", "1"}], ";"}], 
   "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{"(*", 
    RowBox[{"euler", "\[IndentingNewLine]", "cayley"}], "*)"}], 
   "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
    "Now", " ", "define", " ", "conversion", " ", "functions", " ", "for", 
     " ", "each", " ", "representation", " ", "to", " ", "rotation", " ", 
     "matrix"}], "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"quat2rotMat", "[", 
      RowBox[{"{", 
       RowBox[{"qx_", ",", "qy_", ",", "qz_", ",", "qw_"}], "}"}], "]"}], "=", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"1", "-", 
           RowBox[{"2", "*", "qy"}], "-", 
           RowBox[{"2", "*", "qz"}]}], ",", "  ", 
          RowBox[{"2", "*", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"qx", "*", "qy"}], "+", 
             RowBox[{"qz", "*", "qw"}]}], ")"}]}], ",", " ", 
          RowBox[{"2", "*", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"qx", "*", "qz"}], "-", 
             RowBox[{"qy", "*", "qw"}]}], ")"}]}]}], "}"}], ",", 
        "\[IndentingNewLine]", "   ", 
        RowBox[{"{", " ", 
         RowBox[{
          RowBox[{"2", "*", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"qx", "*", "qy"}], "-", 
             RowBox[{"qz", "*", "qw"}]}], ")"}]}], ",", 
          RowBox[{"1", "-", 
           RowBox[{"2", "*", "qx"}], "-", 
           RowBox[{"2", "*", "qz"}]}], ",", "           ", 
          RowBox[{"2", "*", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"qy", "*", "qz"}], "+", 
             RowBox[{"qx", "*", "qw"}]}], ")"}]}]}], "}"}], ",", 
        "\[IndentingNewLine]", "   ", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"2", "*", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"qx", "*", "qz"}], "+", 
             RowBox[{"qy", "*", "qw"}]}], ")"}]}], ",", " ", 
          RowBox[{"2", "*", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"qy", "*", "qz"}], "-", 
             RowBox[{"q1", "*", "qw"}]}], ")"}]}], ",", "          ", 
          RowBox[{"1", "-", 
           RowBox[{"2", "*", "qx"}], "-", 
           RowBox[{"2", "*", "qy"}]}]}], "}"}]}], "}"}], " ", "//", 
      "MatrixForm"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"R", "==", 
     RowBox[{"quat2rotMat", "[", "quat", "]"}]}], " ", ";"}], 
   "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"rotVec2rotMat", "[", 
      RowBox[{"{", 
       RowBox[{"rv1_", ",", "rv2_", ",", "rv3_"}], "}"}], "]"}], "=", 
     RowBox[{
      RowBox[{"Module", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
         "rvm", ",", "rotVecN", ",", "rv1n", ",", "rv2n", ",", "rv3n"}], 
         "}"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"rvm", "=", 
          RowBox[{"Sqrt", "[", 
           RowBox[{
            RowBox[{"rv1", "*", "rv1"}], " ", "+", " ", 
            RowBox[{"rv2", "*", "rv2"}], " ", "+", " ", 
            RowBox[{"rv3", "*", "rv3"}]}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"rotVecN", "=", 
          RowBox[{"rotVec", "/", "rvm"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"rv1n", "=", 
          RowBox[{"rotVecN", "[", 
           RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"rv2n", "=", 
          RowBox[{"rotVecN", "[", 
           RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"rv3n", "=", 
          RowBox[{"rotVecN", "[", 
           RowBox[{"[", "1", "]"}], "]"}]}], ";", " ", "\[IndentingNewLine]", 
         
         RowBox[{"sinrv", "=", 
          RowBox[{"Sin", "[", "rvm", "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"cosrv", "=", 
          RowBox[{"Cos", "[", "rvm", "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"tanrv", "=", 
          RowBox[{"1", "-", "cosrv"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{
              RowBox[{"tanrv", "*", "rv1n", "*", "rv1n"}], "+", "cosrv"}], 
             ",", 
             RowBox[{
              RowBox[{"tanrv", "*", "rv1n", "*", "rv2n"}], "-", 
              RowBox[{"sinrv", "*", "rv3n"}]}], ",", 
             RowBox[{
              RowBox[{"tanrv", "*", "rv1n", "*", "rv3n"}], "+", 
              RowBox[{"sinrv", "*", "rv2n"}]}]}], "}"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"{", " ", 
            RowBox[{
             RowBox[{
              RowBox[{"tanrv", "*", "rv1n", "*", "rv2n"}], "+", 
              RowBox[{"sinrv", "*", "rv3n"}]}], ",", 
             RowBox[{
              RowBox[{"tanrv", "*", "rv2n", "*", "rv2n"}], "+", "cosrv"}], 
             ",", 
             RowBox[{
              RowBox[{"tanrv", "*", "rv2n", "*", "rv3n"}], "-", 
              RowBox[{"sinrv", "*", "rv1n"}]}]}], "}"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{
              RowBox[{"tanrv", "*", "rv1n", "*", "rv3n"}], "-", 
              RowBox[{"sinrv", "*", "rv2n"}]}], ",", 
             RowBox[{
              RowBox[{"tanrv", "*", "rv2n", "*", "rv3n"}], "+", 
              RowBox[{"sinrv", "*", "rv1n"}]}], ",", 
             RowBox[{
              RowBox[{"tanrv", "*", "rv3n", "*", "rv3n"}], "+", "cosrv"}]}], 
            "}"}]}], "}"}]}]}], "\[IndentingNewLine]", "]"}], " ", "//", 
      "MatrixForm"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"R", "==", 
     RowBox[{"rotVec2rotMat", "[", "rotVec", "]"}]}], ";"}], 
   "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"aa2rotMat", "[", 
      RowBox[{"{", 
       RowBox[{"aa1_", ",", " ", "aa2_", ",", " ", "aa3_", ",", " ", "aa4_"}],
        "}"}], "]"}], "=", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"1", "-", 
           RowBox[{
            RowBox[{"Cos", "[", "aa4", "]"}], "*", "aa1", "*", "aa1"}], "+", 
           "caa"}], ",", 
          RowBox[{"1", "-", 
           RowBox[{
            RowBox[{"Cos", "[", "aa4", "]"}], "*", "aa1", "*", "aa2"}], "-", 
           RowBox[{
            RowBox[{"Sin", "[", "aa4", "]"}], "*", "aa3"}]}], ",", 
          RowBox[{"1", "-", 
           RowBox[{
            RowBox[{"Cos", "[", "aa4", "]"}], "*", "aa1", "*", "aa3"}], "+", 
           RowBox[{
            RowBox[{"Sin", "[", "aa4", "]"}], "*", "aa2"}]}]}], "}"}], ",", 
        "\[IndentingNewLine]", "   ", 
        RowBox[{"{", " ", 
         RowBox[{
          RowBox[{"1", "-", 
           RowBox[{
            RowBox[{"Cos", "[", "aa4", "]"}], "*", "aa1", "*", "aa2"}], "+", 
           RowBox[{
            RowBox[{"Sin", "[", "aa4", "]"}], "*", "aa3"}]}], ",", 
          RowBox[{"1", "-", 
           RowBox[{
            RowBox[{"Cos", "[", "aa4", "]"}], "*", "aa2", "*", "aa2"}], "+", 
           RowBox[{"Cos", "[", "aa4", "]"}]}], ",", 
          RowBox[{"1", "-", 
           RowBox[{
            RowBox[{"Cos", "[", "aa4", "]"}], "*", "aa2", "*", "aa3"}], "-", 
           RowBox[{
            RowBox[{"Sin", "[", "aa4", "]"}], "*", "aa1"}]}]}], "}"}], ",", 
        "\[IndentingNewLine]", "   ", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"1", "-", 
           RowBox[{
            RowBox[{"Cos", "[", "aa4", "]"}], "*", "aa1", "*", "aa3"}], "-", 
           RowBox[{
            RowBox[{"Sin", "[", "aa4", "]"}], "*", "aa2"}]}], ",", 
          RowBox[{"1", "-", 
           RowBox[{
            RowBox[{"Cos", "[", "aa4", "]"}], "*", "aa2", "*", "aa3"}], "+", 
           RowBox[{
            RowBox[{"Sin", "[", "aa4", "]"}], "*", "aa1"}]}], ",", 
          RowBox[{"1", "-", 
           RowBox[{
            RowBox[{"Cos", "[", "aa4", "]"}], "*", "aa3", "*", "aa3"}], "+", 
           RowBox[{"Cos", "[", "aa4", "]"}]}]}], "}"}]}], "}"}], " ", "//", 
      "MatrixForm"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"R", "==", 
     RowBox[{"aa2rotMat", "[", "angleAxis", "]"}]}], ";"}], 
   "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
    "Now", " ", "define", " ", "the", " ", "inverse", " ", "conversion", " ", 
     "functions", " ", "from", " ", "rot", " ", "mat", " ", "to", " ", 
     "everything"}], "*)"}], "\[IndentingNewLine]", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"rotMat2quat", "[", "rotMat_", "]"}], "=", 
     RowBox[{"Module", "[", 
      RowBox[{
       RowBox[{"{", "qi", "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"qi", "=", 
         RowBox[{
          RowBox[{"Sqrt", "[", 
           RowBox[{"1", "+", 
            RowBox[{"rotMat", "[", 
             RowBox[{"1", ",", "1"}], "]"}], "-", 
            RowBox[{"rotMat", "[", 
             RowBox[{"2", ",", "2"}], "]"}], "-", 
            RowBox[{"rotMat", "[", 
             RowBox[{"3", ",", "3"}], "]"}]}], "]"}], "/", "2"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{"qi", ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"rotMat", "[", 
              RowBox[{"1", ",", "2"}], "]"}], " ", "+", " ", 
             RowBox[{"rotMat", "[", 
              RowBox[{"2", ",", "1"}], "]"}]}], ")"}], "/", 
           RowBox[{"(", 
            RowBox[{"4", "*", "qi"}], ")"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"rotMat", "[", 
              RowBox[{"1", ",", "3"}], "]"}], " ", "+", " ", 
             RowBox[{"rotMat", "[", 
              RowBox[{"3", ",", "1"}], "]"}]}], ")"}], "/", 
           RowBox[{"(", 
            RowBox[{"4", "*", "qi"}], ")"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"rotMat", "[", 
              RowBox[{"3", ",", "2"}], "]"}], " ", "-", " ", 
             RowBox[{"rotMat", "[", 
              RowBox[{"2", ",", "3"}], "]"}]}], ")"}], "/", 
           RowBox[{"(", 
            RowBox[{"4", "*", "qi"}], ")"}]}]}], "\[IndentingNewLine]", 
         "}"}]}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"quat", "==", 
     RowBox[{"rotMat2quat", "[", "R", "]"}]}], ";"}], "\[IndentingNewLine]", 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"rotMat2rotVec", "[", "rotMat_", "]"}], "=", 
     RowBox[{"Module", "[", 
      RowBox[{
       RowBox[{"{", "theta", "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"theta", "=", 
         RowBox[{"ArcCos", "[", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"rotMat", "[", 
              RowBox[{"1", ",", "1"}], "]"}], "+", 
             RowBox[{"rotMat", "[", 
              RowBox[{"2", ",", "2"}], "]"}], "+", 
             RowBox[{"rotMat", "[", 
              RowBox[{"3", ",", "3"}], "]"}], "-", "1"}], ")"}], "/", "2"}], 
          "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"theta", "*", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{"rotMat", "[", 
               RowBox[{"3", ",", "2"}], "]"}], "-", 
              RowBox[{"rotMat", "[", 
               RowBox[{"2", ",", "3"}], "]"}]}], ")"}], "/", 
            RowBox[{"(", 
             RowBox[{"2", "*", 
              RowBox[{"Sin", "[", "theta", "]"}]}], ")"}]}]}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"theta", "*", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{"rotMat", "[", 
               RowBox[{"1", ",", "3"}], "]"}], "-", 
              RowBox[{"rotMat", "[", 
               RowBox[{"3", ",", "1"}], "]"}]}], ")"}], "/", 
            RowBox[{"(", 
             RowBox[{"2", "*", 
              RowBox[{"Sin", "[", "theta", "]"}]}], ")"}]}]}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"theta", "*", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{"rotMat", "[", 
               RowBox[{"2", ",", "1"}], "]"}], "-", 
              RowBox[{"rotMat", "[", 
               RowBox[{"1", ",", "2"}], "]"}]}], ")"}], "/", 
            RowBox[{"(", 
             RowBox[{"2", "*", 
              RowBox[{"Sin", "[", "theta", "]"}]}], ")"}]}]}]}], 
         "\[IndentingNewLine]", "}"}]}]}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"rotVec", "==", 
     RowBox[{"rotMat2rotVec", "[", "R", "]"}]}], ";"}], "\[IndentingNewLine]",
    "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"rotMat2aa", "[", "rotMat_", "]"}], "=", 
     RowBox[{"Module", "[", 
      RowBox[{
       RowBox[{"{", "theta", "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"theta", "=", 
         RowBox[{"ArcCos", "[", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"rotMat", "[", 
              RowBox[{"1", ",", "1"}], "]"}], "+", 
             RowBox[{"rotMat", "[", 
              RowBox[{"2", ",", "2"}], "]"}], "+", 
             RowBox[{"rotMat", "[", 
              RowBox[{"3", ",", "3"}], "]"}], "-", "1"}], ")"}], "/", "2"}], 
          "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"rotMat", "[", 
              RowBox[{"3", ",", "2"}], "]"}], "-", 
             RowBox[{"rotMat", "[", 
              RowBox[{"2", ",", "3"}], "]"}]}], ")"}], "/", 
           RowBox[{"(", 
            RowBox[{"2", "*", 
             RowBox[{"Sin", "[", "theta", "]"}]}], ")"}]}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"rotMat", "[", 
              RowBox[{"1", ",", "3"}], "]"}], "-", 
             RowBox[{"rotMat", "[", 
              RowBox[{"3", ",", "1"}], "]"}]}], ")"}], "/", 
           RowBox[{"(", 
            RowBox[{"2", "*", 
             RowBox[{"Sin", "[", "theta", "]"}]}], ")"}]}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"rotMat", "[", 
              RowBox[{"2", ",", "1"}], "]"}], "-", 
             RowBox[{"rotMat", "[", 
              RowBox[{"1", ",", "2"}], "]"}]}], ")"}], "/", 
           RowBox[{"(", 
            RowBox[{"2", "*", 
             RowBox[{"Sin", "[", "theta", "]"}]}], ")"}]}], ",", 
          "\[IndentingNewLine]", "theta"}], "}"}]}]}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"angleAxis", "==", 
     RowBox[{"rotMat2aa", "[", "R", "]"}]}], ";"}], "\[IndentingNewLine]", 
   "\[IndentingNewLine]", 
   RowBox[{"(*", 
    RowBox[{
     RowBox[{
     "finally", " ", "define", " ", "the", " ", "conversion", " ", 
      "functions", " ", "between", " ", "different", " ", "representations"}],
      ",", " ", 
     RowBox[{
     "going", " ", "via", " ", "rot", " ", "mat", " ", "when", " ", 
      "necessary"}]}], "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"quat2rotVec", "[", "quat_", "]"}], "=", 
     RowBox[{"Module", "[", 
      RowBox[{
       RowBox[{"{", "normaliser", "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"normaliser", "=", 
         RowBox[{"2", "*", 
          RowBox[{
           RowBox[{"ArcCos", "[", 
            RowBox[{"quat", "[", 
             RowBox[{"[", "4", "]"}], "]"}], "]"}], "/", 
           RowBox[{"Sqrt", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"quat", "[", 
               RowBox[{"[", "1", "]"}], "]"}], "*", 
              RowBox[{"quat", "[", 
               RowBox[{"[", "1", "]"}], "]"}]}], "+", 
             RowBox[{
              RowBox[{"quat", "[", 
               RowBox[{"[", "2", "]"}], "]"}], "*", 
              RowBox[{"quat", "[", 
               RowBox[{"[", "2", "]"}], "]"}]}], "+", 
             RowBox[{
              RowBox[{"quat", "[", 
               RowBox[{"[", "3", "]"}], "]"}], "*", 
              RowBox[{"quat", "[", 
               RowBox[{"[", "3", "]"}], "]"}]}]}], "]"}]}]}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"quat", "[", 
            RowBox[{"[", "1", "]"}], "]"}], "*", "normaliser"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"quat", "[", 
            RowBox[{"[", "2", "]"}], "]"}], "*", "normaliser"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"quat", "[", 
            RowBox[{"[", "3", "]"}], "]"}], "*", "normaliser"}]}], "}"}]}]}], 
      "\[IndentingNewLine]", "]"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"rotVec", "==", 
     RowBox[{"quat2rotVec", "[", "quat", "]"}]}], ";"}], 
   "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"quat2aa", "[", "quat_", "]"}], "=", 
     RowBox[{"Module", "[", 
      RowBox[{
       RowBox[{"{", "qmag", "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"qmag", "=", 
         RowBox[{"Sqrt", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"quat", "[", 
             RowBox[{"[", "1", "]"}], "]"}], "*", 
            RowBox[{"quat", "[", 
             RowBox[{"[", "1", "]"}], "]"}]}], "+", 
           RowBox[{
            RowBox[{"quat", "[", 
             RowBox[{"[", "2", "]"}], "]"}], "*", 
            RowBox[{"quat", "[", 
             RowBox[{"[", "2", "]"}], "]"}]}], "+", 
           RowBox[{
            RowBox[{"quat", "[", 
             RowBox[{"[", "3", "]"}], "]"}], "*", 
            RowBox[{"quat", "[", 
             RowBox[{"[", "3", "]"}], "]"}]}]}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"quat", "[", 
            RowBox[{"[", "1", "]"}], "]"}], "/", "qmag"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"quat", "[", 
            RowBox[{"[", "2", "]"}], "]"}], "/", "qmag"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"quat", "[", 
            RowBox[{"[", "3", "]"}], "]"}], "/", "qmag"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"2", "*", 
           RowBox[{"ArcCos", "[", 
            RowBox[{"quat", "[", 
             RowBox[{"[", "4", "]"}], "]"}], "]"}]}]}], "}"}]}]}], 
      "\[IndentingNewLine]", "]"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"angleAxis", "==", 
     RowBox[{"quat2aa", "[", "quat", "]"}]}], ";"}], "\[IndentingNewLine]", 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"rotVec2quat", "[", "rotVec_", "]"}], "=", 
     RowBox[{"Module", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"rvm", ",", "rotVecN"}], "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"rvm", "=", 
         RowBox[{"Sqrt", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"rotVec", "[", 
             RowBox[{"[", "1", "]"}], "]"}], "*", 
            RowBox[{"rotVec", "[", 
             RowBox[{"[", "1", "]"}], "]"}]}], " ", "+", " ", 
           RowBox[{
            RowBox[{"rotVec", "[", 
             RowBox[{"[", "2", "]"}], "]"}], "*", 
            RowBox[{"rotVec", "[", 
             RowBox[{"[", "2", "]"}], "]"}]}], " ", "+", " ", 
           RowBox[{
            RowBox[{"rotVec", "[", 
             RowBox[{"[", "3", "]"}], "]"}], "*", 
            RowBox[{"rotVec", "[", 
             RowBox[{"[", "3", "]"}], "]"}]}]}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"rotVecN", "=", 
         RowBox[{"rotVec", "/", "rvm"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"rotVecN", "[", 
            RowBox[{"[", "1", "]"}], "]"}], "*", 
           RowBox[{"Sin", "[", 
            RowBox[{"rvm", "/", "2"}], "]"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"rotVecN", "[", 
            RowBox[{"[", "2", "]"}], "]"}], "*", 
           RowBox[{"Sin", "[", 
            RowBox[{"rvm", "/", "2"}], "]"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"rotVecN", "[", 
            RowBox[{"[", "3", "]"}], "]"}], "*", 
           RowBox[{"Sin", "[", 
            RowBox[{"rvm", "/", "2"}], "]"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{"Cos", "[", 
           RowBox[{"rvm", "/", "2"}], "]"}]}], "}"}]}]}], 
      "\[IndentingNewLine]", "]"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"quat", "==", 
     RowBox[{"rotVec2quat", "[", "rotVec", "]"}]}], ";"}], 
   "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"rotVec2aa", "[", "rotVec_", "]"}], "=", 
     RowBox[{"Module", "[", 
      RowBox[{
       RowBox[{"{", "rvm", "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"rvm", "=", 
         RowBox[{"Sqrt", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"rotVec", "[", 
             RowBox[{"[", "1", "]"}], "]"}], "*", 
            RowBox[{"rotVec", "[", 
             RowBox[{"[", "1", "]"}], "]"}]}], " ", "+", " ", 
           RowBox[{
            RowBox[{"rotVec", "[", 
             RowBox[{"[", "2", "]"}], "]"}], "*", 
            RowBox[{"rotVec", "[", 
             RowBox[{"[", "2", "]"}], "]"}]}], " ", "+", " ", 
           RowBox[{
            RowBox[{"rotVec", "[", 
             RowBox[{"[", "3", "]"}], "]"}], "*", 
            RowBox[{"rotVec", "[", 
             RowBox[{"[", "3", "]"}], "]"}]}]}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"rotVec", "[", 
            RowBox[{"[", "1", "]"}], "]"}], "/", "rvm"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"rotVec", "[", 
            RowBox[{"[", "2", "]"}], "]"}], "/", "rvm"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"rotVec", "[", 
            RowBox[{"[", "3", "]"}], "]"}], "/", "rvm"}], ",", 
          "\[IndentingNewLine]", "rvm"}], "}"}]}]}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"angleAxis", "==", 
     RowBox[{"rotVec2quat", "[", "rotVec", "]"}]}], ";"}], 
   "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"aa2quat", "[", "angleAxis_", "]"}], "=", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"angleAxis", "[", 
         RowBox[{"[", "1", "]"}], "]"}], "*", 
        RowBox[{"Sin", "[", 
         RowBox[{
          RowBox[{"angleAxis", "[", 
           RowBox[{"[", "4", "]"}], "]"}], "/", "2"}], "]"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"angleAxis", "[", 
         RowBox[{"[", "2", "]"}], "]"}], "*", 
        RowBox[{"Sin", "[", 
         RowBox[{
          RowBox[{"angleAxis", "[", 
           RowBox[{"[", "4", "]"}], "]"}], "/", "2"}], "]"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"angleAxis", "[", 
         RowBox[{"[", "3", "]"}], "]"}], "*", 
        RowBox[{"Sin", "[", 
         RowBox[{
          RowBox[{"angleAxis", "[", 
           RowBox[{"[", "4", "]"}], "]"}], "/", "2"}], "]"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Cos", "[", 
        RowBox[{
         RowBox[{"angleAxis", "[", 
          RowBox[{"[", "4", "]"}], "]"}], "/", "2"}], "]"}]}], "}"}]}], ";"}],
    "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"quat", "==", 
     RowBox[{"aa2quat", "[", "angleAxis", "]"}]}], ";"}], 
   "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"aa2rotVec", "[", "angleAxis_", "]"}], "=", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"angleAxis", "[", 
         RowBox[{"[", "1", "]"}], "]"}], "*", 
        RowBox[{"angleAxis", "[", 
         RowBox[{"[", "4", "]"}], "]"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"angleAxis", "[", 
         RowBox[{"[", "2", "]"}], "]"}], "*", 
        RowBox[{"angleAxis", "[", 
         RowBox[{"[", "4", "]"}], "]"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"angleAxis", "[", 
         RowBox[{"[", "3", "]"}], "]"}], "*", 
        RowBox[{"angleAxis", "[", 
         RowBox[{"[", "4", "]"}], "]"}]}]}], "}"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"rotVec", "==", 
     RowBox[{"aa2rotVec", "[", "angleAxis", "]"}]}], ";"}], 
   "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
    "Now", " ", "calculate", " ", "the", " ", "jacobians", " ", "of", " ", 
     "these", " ", "conversion", " ", "functions"}], "*)"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"Jquat2rotVec", "[", 
      RowBox[{"{", 
       RowBox[{"qx_", ",", "qy_", ",", "qz_", ",", "qw_"}], "}"}], "]"}], " ",
      "=", " ", 
     RowBox[{"D", "[", 
      RowBox[{
       RowBox[{"quat2rotVec", "[", 
        RowBox[{"{", 
         RowBox[{"qx", ",", "qy", ",", "qz", ",", "qw"}], "}"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"qx", ",", "qy", ",", "qz", ",", "qw"}], "}"}], ",", "2"}], 
        "}"}]}], "]"}]}], " ", ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Jquat2rotVec", "[", "quat", "]"}], " ", "//", "MatrixForm"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"Jquat2aa", "[", 
      RowBox[{"{", 
       RowBox[{"qx_", ",", "qy_", ",", "qz_", ",", "qw_"}], "}"}], "]"}], " ",
      "=", " ", 
     RowBox[{"D", "[", 
      RowBox[{
       RowBox[{"quat2aa", "[", 
        RowBox[{"{", 
         RowBox[{"qx", ",", "qy", ",", "qz", ",", "qw"}], "}"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"qx", ",", "qy", ",", "qz", ",", "qw"}], "}"}], ",", "2"}], 
        "}"}]}], "]"}]}], " ", ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Jquat2aa", "[", "quat", "]"}], "//", "MatrixForm"}], 
   "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"JrotVec2quat", "[", 
      RowBox[{"{", 
       RowBox[{"rv1_", ",", "rv2_", ",", "rv3_"}], "}"}], "]"}], " ", "=", 
     " ", 
     RowBox[{"D", "[", 
      RowBox[{
       RowBox[{"rotVec2quat", "[", 
        RowBox[{"{", 
         RowBox[{"rv1", ",", "rv2", ",", "rv3"}], "}"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"rv1", ",", "rv2", ",", "rv3"}], "}"}], ",", "2"}], "}"}]}],
       "]"}]}], " ", ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"JrotVec2quat", "[", "rotVec", "]"}], "//", "MatrixForm"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"JrotVec2aa", "[", 
      RowBox[{"{", 
       RowBox[{"rv1_", ",", "rv2_", ",", "rv3_"}], "}"}], "]"}], " ", "=", 
     " ", 
     RowBox[{"D", "[", 
      RowBox[{
       RowBox[{"rotVec2aa", "[", 
        RowBox[{"{", 
         RowBox[{"rv1", ",", "rv2", ",", "rv3"}], "}"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"rv1", ",", "rv2", ",", "rv3"}], "}"}], ",", "2"}], "}"}]}],
       "]"}]}], " ", ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"JrotVec2aa", "[", "rotVec", "]"}], "//", "MatrixForm"}], 
   "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"Jaa2quat", "[", 
      RowBox[{"{", 
       RowBox[{"aa1_", ",", "aa2_", ",", "aa3_", ",", "aa4_"}], "}"}], "]"}], 
     " ", "=", " ", 
     RowBox[{"D", "[", 
      RowBox[{
       RowBox[{"aa2quat", "[", 
        RowBox[{"{", 
         RowBox[{"aa1", ",", "aa2", ",", "aa3", ",", "aa4"}], "}"}], "]"}], 
       ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"aa1", ",", "aa2", ",", "aa3", ",", "aa4"}], "}"}], ",", 
         "2"}], "}"}]}], "]"}]}], " ", ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Jaa2quat", "[", "angleAxis", "]"}], " ", "//", "MatrixForm"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"Jaa2rotVec", "[", 
      RowBox[{"{", 
       RowBox[{"aa1_", ",", "aa2_", ",", "aa3_", ",", "aa4_"}], "}"}], "]"}], 
     " ", "=", " ", 
     RowBox[{"D", "[", 
      RowBox[{
       RowBox[{"aa2rotVec", "[", 
        RowBox[{"{", 
         RowBox[{"aa1", ",", "aa2", ",", "aa3", ",", "aa4"}], "}"}], "]"}], 
       ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"aa1", ",", "aa2", ",", "aa3", ",", "aa4"}], "}"}], ",", 
         "2"}], "}"}]}], "]"}]}], " ", ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Jaa2rotVec", "[", "angleAxis", "]"}], " ", "//", "MatrixForm"}], 
   "\[IndentingNewLine]"}]}]], "Input",
 CellChangeTimes->{{3.452437981767764*^9, 3.452437996995854*^9}, {
   3.4524406489812927`*^9, 3.452440652067245*^9}, {3.452446782440468*^9, 
   3.452446787270488*^9}, {3.5787274473364534`*^9, 3.5787274726096535`*^9}, {
   3.5787294255289087`*^9, 3.57872944055081*^9}, {3.5787305228958435`*^9, 
   3.5787305294546742`*^9}, {3.578732009207023*^9, 3.578732031206808*^9}, {
   3.5787342739027576`*^9, 3.578734287534484*^9}, 3.578734333518306*^9, {
   3.5787368529892907`*^9, 3.5787368543324604`*^9}, {3.585898412146451*^9, 
   3.5858985230576377`*^9}, {3.5858986549695096`*^9, 3.585898667342092*^9}, 
   3.603624835906171*^9, {3.603698394384819*^9, 3.6036983989490805`*^9}, {
   3.603698430710905*^9, 3.603698437173276*^9}, 3.6393257758046207`*^9, 
   3.639387451781255*^9, {3.6393875290387163`*^9, 3.6393876846047006`*^9}, {
   3.6393877345455847`*^9, 3.6393878052696695`*^9}, {3.639387872483551*^9, 
   3.6393879207743397`*^9}, {3.6393879933025284`*^9, 
   3.6393879957796717`*^9}, {3.639388257474785*^9, 3.6393882670683393`*^9}, {
   3.6393883178542724`*^9, 3.639388468746987*^9}, {3.6393890253701324`*^9, 
   3.639389090003866*^9}, {3.639389124226842*^9, 3.639389161524996*^9}, {
   3.6393893075704308`*^9, 3.639389322605299*^9}, {3.639389402857933*^9, 
   3.6393894828995557`*^9}, {3.639389516892519*^9, 3.6393895369536777`*^9}, {
   3.63990591996638*^9, 3.6399059254836917`*^9}, {3.639908274685561*^9, 
   3.639908289105375*^9}, {3.6399903571878195`*^9, 3.639990361018036*^9}, {
   3.640107541419482*^9, 3.640107978382841*^9}, {3.640108013870723*^9, 
   3.640108033632083*^9}, {3.640108065250688*^9, 3.6401080689798164`*^9}, {
   3.6401081298165035`*^9, 3.640108395068584*^9}, {3.6401085381484003`*^9, 
   3.6401085383635225`*^9}, {3.6401085950778975`*^9, 3.640108791100794*^9}, {
   3.640109003679947*^9, 3.640109300380311*^9}, {3.6401093403381195`*^9, 
   3.6401094357755985`*^9}, {3.6401110843108025`*^9, 3.64011108441286*^9}, {
   3.6401111166392555`*^9, 3.6401111757640057`*^9}, {3.6401112564120426`*^9, 
   3.64011125859629*^9}, {3.6401113010375233`*^9, 3.6401113411574163`*^9}, {
   3.6401113719139743`*^9, 3.6401113959617014`*^9}, {3.6401114332589912`*^9, 
   3.640111456046999*^9}, {3.6401115053051176`*^9, 3.640111819812648*^9}, {
   3.640111906020857*^9, 3.640111958155618*^9}, {3.64011215967365*^9, 
   3.6401121598057256`*^9}, {3.6401122585610967`*^9, 
   3.6401122716405635`*^9}, {3.6401125329026995`*^9, 
   3.6401125430214753`*^9}, {3.6401125746295185`*^9, 3.640112582582057*^9}, {
   3.640112619471114*^9, 3.640112858635637*^9}, {3.6401128901536283`*^9, 
   3.640113082933673*^9}, {3.640113125251829*^9, 3.640113139040699*^9}, {
   3.6401131774646335`*^9, 3.640113253001752*^9}, {3.6401132927064166`*^9, 
   3.6401134460969753`*^9}, {3.640113486254899*^9, 3.64011353861679*^9}, {
   3.6401136330236793`*^9, 3.6401139575669374`*^9}, {3.6401139931492486`*^9, 
   3.6401141868728323`*^9}, {3.6401142665232983`*^9, 3.640114456953001*^9}, {
   3.640114488012731*^9, 3.640114553737248*^9}, {3.640114614422889*^9, 
   3.640115044608452*^9}, {3.6401150815365305`*^9, 3.640115131110831*^9}, {
   3.6401151961839757`*^9, 3.6401152887207975`*^9}, {3.6401154023746743`*^9, 
   3.640115433778601*^9}, {3.6401154789463844`*^9, 3.640115688560033*^9}, {
   3.6401523196648326`*^9, 3.6401523462210083`*^9}, {3.6401523908798103`*^9, 
   3.6401523957955503`*^9}, {3.6401526263351383`*^9, 
   3.6401526268595495`*^9}, {3.640152738379173*^9, 3.6401527816677914`*^9}, {
   3.6401530470411425`*^9, 3.640153128471808*^9}, {3.640154254227536*^9, 
   3.640154278789012*^9}, {3.640154763962309*^9, 3.6401547767753186`*^9}, {
   3.64015481354348*^9, 3.640154815574479*^9}, {3.640155091502006*^9, 
   3.6401551041366353`*^9}, 3.640155262773436*^9, 3.640155502520983*^9, 
   3.640155594585964*^9, 3.640155666929967*^9, {3.6401557625353565`*^9, 
   3.640155768552517*^9}, {3.6401558052084184`*^9, 3.6401558203277082`*^9}, {
   3.7203553828272405`*^9, 3.7203553875672407`*^9}, 3.7207914857907543`*^9, {
   3.720791531575754*^9, 3.7207915703557544`*^9}}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {
           RowBox[{
            FractionBox[
             RowBox[{"6", " ", 
              SuperscriptBox["qx", "3"], " ", 
              RowBox[{"ArcCos", "[", "qw", "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"6", " ", "qx", " ", 
              RowBox[{"ArcCos", "[", "qw", "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           RowBox[{
            FractionBox[
             RowBox[{"6", " ", 
              SuperscriptBox["qx", "2"], " ", "qy", " ", 
              RowBox[{"ArcCos", "[", "qw", "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"2", " ", "qy", " ", 
              RowBox[{"ArcCos", "[", "qw", "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           RowBox[{
            FractionBox[
             RowBox[{"6", " ", 
              SuperscriptBox["qx", "2"], " ", "qz", " ", 
              RowBox[{"ArcCos", "[", "qw", "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"2", " ", "qz", " ", 
              RowBox[{"ArcCos", "[", "qw", "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           RowBox[{
            FractionBox[
             RowBox[{"2", " ", 
              SuperscriptBox["qx", "2"]}], 
             RowBox[{
              SqrtBox[
               RowBox[{"1", "-", 
                SuperscriptBox["qw", "2"]}]], " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["qx", "2"], "+", 
                 SuperscriptBox["qy", "2"], "+", 
                 SuperscriptBox["qz", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]]}]], "-", 
            FractionBox["2", 
             RowBox[{
              SqrtBox[
               RowBox[{"1", "-", 
                SuperscriptBox["qw", "2"]}]], " ", 
              SqrtBox[
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}]]}]]}]}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {
           RowBox[{
            FractionBox[
             RowBox[{"6", " ", 
              SuperscriptBox["qx", "2"], " ", "qy", " ", 
              RowBox[{"ArcCos", "[", "qw", "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"2", " ", "qy", " ", 
              RowBox[{"ArcCos", "[", "qw", "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           RowBox[{
            FractionBox[
             RowBox[{"6", " ", "qx", " ", 
              SuperscriptBox["qy", "2"], " ", 
              RowBox[{"ArcCos", "[", "qw", "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"2", " ", "qx", " ", 
              RowBox[{"ArcCos", "[", "qw", "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           FractionBox[
            RowBox[{"6", " ", "qx", " ", "qy", " ", "qz", " ", 
             RowBox[{"ArcCos", "[", "qw", "]"}]}], 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox["qx", "2"], "+", 
               SuperscriptBox["qy", "2"], "+", 
               SuperscriptBox["qz", "2"]}], ")"}], 
             RowBox[{"5", "/", "2"}]]]},
          {
           FractionBox[
            RowBox[{"2", " ", "qx", " ", "qy"}], 
            RowBox[{
             SqrtBox[
              RowBox[{"1", "-", 
               SuperscriptBox["qw", "2"]}]], " ", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]}]]}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {
           RowBox[{
            FractionBox[
             RowBox[{"6", " ", 
              SuperscriptBox["qx", "2"], " ", "qz", " ", 
              RowBox[{"ArcCos", "[", "qw", "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"2", " ", "qz", " ", 
              RowBox[{"ArcCos", "[", "qw", "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           FractionBox[
            RowBox[{"6", " ", "qx", " ", "qy", " ", "qz", " ", 
             RowBox[{"ArcCos", "[", "qw", "]"}]}], 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox["qx", "2"], "+", 
               SuperscriptBox["qy", "2"], "+", 
               SuperscriptBox["qz", "2"]}], ")"}], 
             RowBox[{"5", "/", "2"}]]]},
          {
           RowBox[{
            FractionBox[
             RowBox[{"6", " ", "qx", " ", 
              SuperscriptBox["qz", "2"], " ", 
              RowBox[{"ArcCos", "[", "qw", "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"2", " ", "qx", " ", 
              RowBox[{"ArcCos", "[", "qw", "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           FractionBox[
            RowBox[{"2", " ", "qx", " ", "qz"}], 
            RowBox[{
             SqrtBox[
              RowBox[{"1", "-", 
               SuperscriptBox["qw", "2"]}]], " ", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]}]]}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {
           RowBox[{
            FractionBox[
             RowBox[{"2", " ", 
              SuperscriptBox["qx", "2"]}], 
             RowBox[{
              SqrtBox[
               RowBox[{"1", "-", 
                SuperscriptBox["qw", "2"]}]], " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["qx", "2"], "+", 
                 SuperscriptBox["qy", "2"], "+", 
                 SuperscriptBox["qz", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]]}]], "-", 
            FractionBox["2", 
             RowBox[{
              SqrtBox[
               RowBox[{"1", "-", 
                SuperscriptBox["qw", "2"]}]], " ", 
              SqrtBox[
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}]]}]]}]},
          {
           FractionBox[
            RowBox[{"2", " ", "qx", " ", "qy"}], 
            RowBox[{
             SqrtBox[
              RowBox[{"1", "-", 
               SuperscriptBox["qw", "2"]}]], " ", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]}]]},
          {
           FractionBox[
            RowBox[{"2", " ", "qx", " ", "qz"}], 
            RowBox[{
             SqrtBox[
              RowBox[{"1", "-", 
               SuperscriptBox["qw", "2"]}]], " ", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]}]]},
          {
           RowBox[{"-", 
            FractionBox[
             RowBox[{"2", " ", "qw", " ", "qx"}], 
             RowBox[{
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{"1", "-", 
                 SuperscriptBox["qw", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]], " ", 
              SqrtBox[
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}]]}]]}]}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}]},
     {
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {
           RowBox[{
            FractionBox[
             RowBox[{"6", " ", 
              SuperscriptBox["qx", "2"], " ", "qy", " ", 
              RowBox[{"ArcCos", "[", "qw", "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"2", " ", "qy", " ", 
              RowBox[{"ArcCos", "[", "qw", "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           RowBox[{
            FractionBox[
             RowBox[{"6", " ", "qx", " ", 
              SuperscriptBox["qy", "2"], " ", 
              RowBox[{"ArcCos", "[", "qw", "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"2", " ", "qx", " ", 
              RowBox[{"ArcCos", "[", "qw", "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           FractionBox[
            RowBox[{"6", " ", "qx", " ", "qy", " ", "qz", " ", 
             RowBox[{"ArcCos", "[", "qw", "]"}]}], 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox["qx", "2"], "+", 
               SuperscriptBox["qy", "2"], "+", 
               SuperscriptBox["qz", "2"]}], ")"}], 
             RowBox[{"5", "/", "2"}]]]},
          {
           FractionBox[
            RowBox[{"2", " ", "qx", " ", "qy"}], 
            RowBox[{
             SqrtBox[
              RowBox[{"1", "-", 
               SuperscriptBox["qw", "2"]}]], " ", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]}]]}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {
           RowBox[{
            FractionBox[
             RowBox[{"6", " ", "qx", " ", 
              SuperscriptBox["qy", "2"], " ", 
              RowBox[{"ArcCos", "[", "qw", "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"2", " ", "qx", " ", 
              RowBox[{"ArcCos", "[", "qw", "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           RowBox[{
            FractionBox[
             RowBox[{"6", " ", 
              SuperscriptBox["qy", "3"], " ", 
              RowBox[{"ArcCos", "[", "qw", "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"6", " ", "qy", " ", 
              RowBox[{"ArcCos", "[", "qw", "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           RowBox[{
            FractionBox[
             RowBox[{"6", " ", 
              SuperscriptBox["qy", "2"], " ", "qz", " ", 
              RowBox[{"ArcCos", "[", "qw", "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"2", " ", "qz", " ", 
              RowBox[{"ArcCos", "[", "qw", "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           RowBox[{
            FractionBox[
             RowBox[{"2", " ", 
              SuperscriptBox["qy", "2"]}], 
             RowBox[{
              SqrtBox[
               RowBox[{"1", "-", 
                SuperscriptBox["qw", "2"]}]], " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["qx", "2"], "+", 
                 SuperscriptBox["qy", "2"], "+", 
                 SuperscriptBox["qz", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]]}]], "-", 
            FractionBox["2", 
             RowBox[{
              SqrtBox[
               RowBox[{"1", "-", 
                SuperscriptBox["qw", "2"]}]], " ", 
              SqrtBox[
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}]]}]]}]}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {
           FractionBox[
            RowBox[{"6", " ", "qx", " ", "qy", " ", "qz", " ", 
             RowBox[{"ArcCos", "[", "qw", "]"}]}], 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox["qx", "2"], "+", 
               SuperscriptBox["qy", "2"], "+", 
               SuperscriptBox["qz", "2"]}], ")"}], 
             RowBox[{"5", "/", "2"}]]]},
          {
           RowBox[{
            FractionBox[
             RowBox[{"6", " ", 
              SuperscriptBox["qy", "2"], " ", "qz", " ", 
              RowBox[{"ArcCos", "[", "qw", "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"2", " ", "qz", " ", 
              RowBox[{"ArcCos", "[", "qw", "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           RowBox[{
            FractionBox[
             RowBox[{"6", " ", "qy", " ", 
              SuperscriptBox["qz", "2"], " ", 
              RowBox[{"ArcCos", "[", "qw", "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"2", " ", "qy", " ", 
              RowBox[{"ArcCos", "[", "qw", "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           FractionBox[
            RowBox[{"2", " ", "qy", " ", "qz"}], 
            RowBox[{
             SqrtBox[
              RowBox[{"1", "-", 
               SuperscriptBox["qw", "2"]}]], " ", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]}]]}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {
           FractionBox[
            RowBox[{"2", " ", "qx", " ", "qy"}], 
            RowBox[{
             SqrtBox[
              RowBox[{"1", "-", 
               SuperscriptBox["qw", "2"]}]], " ", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]}]]},
          {
           RowBox[{
            FractionBox[
             RowBox[{"2", " ", 
              SuperscriptBox["qy", "2"]}], 
             RowBox[{
              SqrtBox[
               RowBox[{"1", "-", 
                SuperscriptBox["qw", "2"]}]], " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["qx", "2"], "+", 
                 SuperscriptBox["qy", "2"], "+", 
                 SuperscriptBox["qz", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]]}]], "-", 
            FractionBox["2", 
             RowBox[{
              SqrtBox[
               RowBox[{"1", "-", 
                SuperscriptBox["qw", "2"]}]], " ", 
              SqrtBox[
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}]]}]]}]},
          {
           FractionBox[
            RowBox[{"2", " ", "qy", " ", "qz"}], 
            RowBox[{
             SqrtBox[
              RowBox[{"1", "-", 
               SuperscriptBox["qw", "2"]}]], " ", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]}]]},
          {
           RowBox[{"-", 
            FractionBox[
             RowBox[{"2", " ", "qw", " ", "qy"}], 
             RowBox[{
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{"1", "-", 
                 SuperscriptBox["qw", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]], " ", 
              SqrtBox[
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}]]}]]}]}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}]},
     {
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {
           RowBox[{
            FractionBox[
             RowBox[{"6", " ", 
              SuperscriptBox["qx", "2"], " ", "qz", " ", 
              RowBox[{"ArcCos", "[", "qw", "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"2", " ", "qz", " ", 
              RowBox[{"ArcCos", "[", "qw", "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           FractionBox[
            RowBox[{"6", " ", "qx", " ", "qy", " ", "qz", " ", 
             RowBox[{"ArcCos", "[", "qw", "]"}]}], 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox["qx", "2"], "+", 
               SuperscriptBox["qy", "2"], "+", 
               SuperscriptBox["qz", "2"]}], ")"}], 
             RowBox[{"5", "/", "2"}]]]},
          {
           RowBox[{
            FractionBox[
             RowBox[{"6", " ", "qx", " ", 
              SuperscriptBox["qz", "2"], " ", 
              RowBox[{"ArcCos", "[", "qw", "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"2", " ", "qx", " ", 
              RowBox[{"ArcCos", "[", "qw", "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           FractionBox[
            RowBox[{"2", " ", "qx", " ", "qz"}], 
            RowBox[{
             SqrtBox[
              RowBox[{"1", "-", 
               SuperscriptBox["qw", "2"]}]], " ", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]}]]}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {
           FractionBox[
            RowBox[{"6", " ", "qx", " ", "qy", " ", "qz", " ", 
             RowBox[{"ArcCos", "[", "qw", "]"}]}], 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox["qx", "2"], "+", 
               SuperscriptBox["qy", "2"], "+", 
               SuperscriptBox["qz", "2"]}], ")"}], 
             RowBox[{"5", "/", "2"}]]]},
          {
           RowBox[{
            FractionBox[
             RowBox[{"6", " ", 
              SuperscriptBox["qy", "2"], " ", "qz", " ", 
              RowBox[{"ArcCos", "[", "qw", "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"2", " ", "qz", " ", 
              RowBox[{"ArcCos", "[", "qw", "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           RowBox[{
            FractionBox[
             RowBox[{"6", " ", "qy", " ", 
              SuperscriptBox["qz", "2"], " ", 
              RowBox[{"ArcCos", "[", "qw", "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"2", " ", "qy", " ", 
              RowBox[{"ArcCos", "[", "qw", "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           FractionBox[
            RowBox[{"2", " ", "qy", " ", "qz"}], 
            RowBox[{
             SqrtBox[
              RowBox[{"1", "-", 
               SuperscriptBox["qw", "2"]}]], " ", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]}]]}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {
           RowBox[{
            FractionBox[
             RowBox[{"6", " ", "qx", " ", 
              SuperscriptBox["qz", "2"], " ", 
              RowBox[{"ArcCos", "[", "qw", "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"2", " ", "qx", " ", 
              RowBox[{"ArcCos", "[", "qw", "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           RowBox[{
            FractionBox[
             RowBox[{"6", " ", "qy", " ", 
              SuperscriptBox["qz", "2"], " ", 
              RowBox[{"ArcCos", "[", "qw", "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"2", " ", "qy", " ", 
              RowBox[{"ArcCos", "[", "qw", "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           RowBox[{
            FractionBox[
             RowBox[{"6", " ", 
              SuperscriptBox["qz", "3"], " ", 
              RowBox[{"ArcCos", "[", "qw", "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"6", " ", "qz", " ", 
              RowBox[{"ArcCos", "[", "qw", "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           RowBox[{
            FractionBox[
             RowBox[{"2", " ", 
              SuperscriptBox["qz", "2"]}], 
             RowBox[{
              SqrtBox[
               RowBox[{"1", "-", 
                SuperscriptBox["qw", "2"]}]], " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["qx", "2"], "+", 
                 SuperscriptBox["qy", "2"], "+", 
                 SuperscriptBox["qz", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]]}]], "-", 
            FractionBox["2", 
             RowBox[{
              SqrtBox[
               RowBox[{"1", "-", 
                SuperscriptBox["qw", "2"]}]], " ", 
              SqrtBox[
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}]]}]]}]}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {
           FractionBox[
            RowBox[{"2", " ", "qx", " ", "qz"}], 
            RowBox[{
             SqrtBox[
              RowBox[{"1", "-", 
               SuperscriptBox["qw", "2"]}]], " ", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]}]]},
          {
           FractionBox[
            RowBox[{"2", " ", "qy", " ", "qz"}], 
            RowBox[{
             SqrtBox[
              RowBox[{"1", "-", 
               SuperscriptBox["qw", "2"]}]], " ", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]}]]},
          {
           RowBox[{
            FractionBox[
             RowBox[{"2", " ", 
              SuperscriptBox["qz", "2"]}], 
             RowBox[{
              SqrtBox[
               RowBox[{"1", "-", 
                SuperscriptBox["qw", "2"]}]], " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["qx", "2"], "+", 
                 SuperscriptBox["qy", "2"], "+", 
                 SuperscriptBox["qz", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]]}]], "-", 
            FractionBox["2", 
             RowBox[{
              SqrtBox[
               RowBox[{"1", "-", 
                SuperscriptBox["qw", "2"]}]], " ", 
              SqrtBox[
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}]]}]]}]},
          {
           RowBox[{"-", 
            FractionBox[
             RowBox[{"2", " ", "qw", " ", "qz"}], 
             RowBox[{
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{"1", "-", 
                 SuperscriptBox["qw", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]], " ", 
              SqrtBox[
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}]]}]]}]}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{{3.6401134239683447`*^9, 3.640113447151578*^9}, 
   3.640113488479169*^9, {3.640113959108817*^9, 3.6401139691115274`*^9}, {
   3.64011501796024*^9, 3.6401150249632382`*^9}, 3.640115132773779*^9, {
   3.6401152031909747`*^9, 3.6401152593180137`*^9}, 3.6401153060386834`*^9, 
   3.640115692595338*^9, 3.6401523477807093`*^9, 3.640152396911435*^9, 
   3.640152628531938*^9, {3.640152754265111*^9, 3.6401527608361225`*^9}, {
   3.640153029892369*^9, 3.6401530736633053`*^9}, {3.6401531197527447`*^9, 
   3.640153129282771*^9}, {3.640154282100628*^9, 3.6401542850546155`*^9}, 
   3.6401548186003137`*^9, 3.6401551060272145`*^9, 3.640155264523883*^9, 
   3.640155503560177*^9, 3.640155595234197*^9, 3.640155668880672*^9, 
   3.640155769232999*^9, {3.6401558100506067`*^9, 3.6401558265389566`*^9}, {
   3.7203552975112405`*^9, 3.7203553086452403`*^9}, 3.7207914868487544`*^9, 
   3.7207915346127543`*^9, 3.7207915713367543`*^9}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {
           RowBox[{
            FractionBox[
             RowBox[{"3", " ", 
              SuperscriptBox["qx", "3"]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"3", " ", "qx"}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           RowBox[{
            FractionBox[
             RowBox[{"3", " ", 
              SuperscriptBox["qx", "2"], " ", "qy"}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox["qy", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           RowBox[{
            FractionBox[
             RowBox[{"3", " ", 
              SuperscriptBox["qx", "2"], " ", "qz"}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox["qz", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {"0"}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {
           RowBox[{
            FractionBox[
             RowBox[{"3", " ", 
              SuperscriptBox["qx", "2"], " ", "qy"}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox["qy", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           RowBox[{
            FractionBox[
             RowBox[{"3", " ", "qx", " ", 
              SuperscriptBox["qy", "2"]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox["qx", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           FractionBox[
            RowBox[{"3", " ", "qx", " ", "qy", " ", "qz"}], 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox["qx", "2"], "+", 
               SuperscriptBox["qy", "2"], "+", 
               SuperscriptBox["qz", "2"]}], ")"}], 
             RowBox[{"5", "/", "2"}]]]},
          {"0"}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {
           RowBox[{
            FractionBox[
             RowBox[{"3", " ", 
              SuperscriptBox["qx", "2"], " ", "qz"}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox["qz", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           FractionBox[
            RowBox[{"3", " ", "qx", " ", "qy", " ", "qz"}], 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox["qx", "2"], "+", 
               SuperscriptBox["qy", "2"], "+", 
               SuperscriptBox["qz", "2"]}], ")"}], 
             RowBox[{"5", "/", "2"}]]]},
          {
           RowBox[{
            FractionBox[
             RowBox[{"3", " ", "qx", " ", 
              SuperscriptBox["qz", "2"]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox["qx", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {"0"}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {"0"},
          {"0"},
          {"0"},
          {"0"}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}]},
     {
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {
           RowBox[{
            FractionBox[
             RowBox[{"3", " ", 
              SuperscriptBox["qx", "2"], " ", "qy"}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox["qy", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           RowBox[{
            FractionBox[
             RowBox[{"3", " ", "qx", " ", 
              SuperscriptBox["qy", "2"]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox["qx", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           FractionBox[
            RowBox[{"3", " ", "qx", " ", "qy", " ", "qz"}], 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox["qx", "2"], "+", 
               SuperscriptBox["qy", "2"], "+", 
               SuperscriptBox["qz", "2"]}], ")"}], 
             RowBox[{"5", "/", "2"}]]]},
          {"0"}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {
           RowBox[{
            FractionBox[
             RowBox[{"3", " ", "qx", " ", 
              SuperscriptBox["qy", "2"]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox["qx", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           RowBox[{
            FractionBox[
             RowBox[{"3", " ", 
              SuperscriptBox["qy", "3"]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"3", " ", "qy"}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           RowBox[{
            FractionBox[
             RowBox[{"3", " ", 
              SuperscriptBox["qy", "2"], " ", "qz"}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox["qz", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {"0"}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {
           FractionBox[
            RowBox[{"3", " ", "qx", " ", "qy", " ", "qz"}], 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox["qx", "2"], "+", 
               SuperscriptBox["qy", "2"], "+", 
               SuperscriptBox["qz", "2"]}], ")"}], 
             RowBox[{"5", "/", "2"}]]]},
          {
           RowBox[{
            FractionBox[
             RowBox[{"3", " ", 
              SuperscriptBox["qy", "2"], " ", "qz"}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox["qz", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           RowBox[{
            FractionBox[
             RowBox[{"3", " ", "qy", " ", 
              SuperscriptBox["qz", "2"]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox["qy", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {"0"}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {"0"},
          {"0"},
          {"0"},
          {"0"}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}]},
     {
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {
           RowBox[{
            FractionBox[
             RowBox[{"3", " ", 
              SuperscriptBox["qx", "2"], " ", "qz"}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox["qz", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           FractionBox[
            RowBox[{"3", " ", "qx", " ", "qy", " ", "qz"}], 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox["qx", "2"], "+", 
               SuperscriptBox["qy", "2"], "+", 
               SuperscriptBox["qz", "2"]}], ")"}], 
             RowBox[{"5", "/", "2"}]]]},
          {
           RowBox[{
            FractionBox[
             RowBox[{"3", " ", "qx", " ", 
              SuperscriptBox["qz", "2"]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox["qx", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {"0"}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {
           FractionBox[
            RowBox[{"3", " ", "qx", " ", "qy", " ", "qz"}], 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox["qx", "2"], "+", 
               SuperscriptBox["qy", "2"], "+", 
               SuperscriptBox["qz", "2"]}], ")"}], 
             RowBox[{"5", "/", "2"}]]]},
          {
           RowBox[{
            FractionBox[
             RowBox[{"3", " ", 
              SuperscriptBox["qy", "2"], " ", "qz"}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox["qz", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           RowBox[{
            FractionBox[
             RowBox[{"3", " ", "qy", " ", 
              SuperscriptBox["qz", "2"]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox["qy", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {"0"}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {
           RowBox[{
            FractionBox[
             RowBox[{"3", " ", "qx", " ", 
              SuperscriptBox["qz", "2"]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox["qx", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           RowBox[{
            FractionBox[
             RowBox[{"3", " ", "qy", " ", 
              SuperscriptBox["qz", "2"]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox["qy", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           RowBox[{
            FractionBox[
             RowBox[{"3", " ", 
              SuperscriptBox["qz", "3"]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"3", " ", "qz"}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["qx", "2"], "+", 
                SuperscriptBox["qy", "2"], "+", 
                SuperscriptBox["qz", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {"0"}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {"0"},
          {"0"},
          {"0"},
          {"0"}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> 
           {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}]},
     {
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {"0"},
          {"0"},
          {"0"},
          {"0"}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {"0"},
          {"0"},
          {"0"},
          {"0"}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {"0"},
          {"0"},
          {"0"},
          {"0"}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {"0"},
          {"0"},
          {"0"},
          {
           RowBox[{"-", 
            FractionBox[
             RowBox[{"2", " ", "qw"}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{"1", "-", 
                SuperscriptBox["qw", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{{3.6401134239683447`*^9, 3.640113447151578*^9}, 
   3.640113488479169*^9, {3.640113959108817*^9, 3.6401139691115274`*^9}, {
   3.64011501796024*^9, 3.6401150249632382`*^9}, 3.640115132773779*^9, {
   3.6401152031909747`*^9, 3.6401152593180137`*^9}, 3.6401153060386834`*^9, 
   3.640115692595338*^9, 3.6401523477807093`*^9, 3.640152396911435*^9, 
   3.640152628531938*^9, {3.640152754265111*^9, 3.6401527608361225`*^9}, {
   3.640153029892369*^9, 3.6401530736633053`*^9}, {3.6401531197527447`*^9, 
   3.640153129282771*^9}, {3.640154282100628*^9, 3.6401542850546155`*^9}, 
   3.6401548186003137`*^9, 3.6401551060272145`*^9, 3.640155264523883*^9, 
   3.640155503560177*^9, 3.640155595234197*^9, 3.640155668880672*^9, 
   3.640155769232999*^9, {3.6401558100506067`*^9, 3.6401558265389566`*^9}, {
   3.7203552975112405`*^9, 3.7203553086452403`*^9}, 3.7207914868487544`*^9, 
   3.7207915346127543`*^9, 3.7207915713907547`*^9}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {
           RowBox[{
            RowBox[{"-", 
             FractionBox[
              RowBox[{"3", " ", 
               SuperscriptBox["rv1", "3"], " ", 
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", 
                 SqrtBox[
                  RowBox[{
                   SuperscriptBox["rv1", "2"], "+", 
                   SuperscriptBox["rv2", "2"], "+", 
                   SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
              RowBox[{"2", " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}], ")"}], "2"]}]]}], "+", 
            FractionBox[
             RowBox[{"3", " ", "rv1", " ", 
              RowBox[{"Cos", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"2", " ", 
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}]}]], "+", 
            FractionBox[
             RowBox[{"3", " ", 
              SuperscriptBox["rv1", "3"], " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"3", " ", "rv1", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{
              SuperscriptBox["rv1", "3"], " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"4", " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]]}]]}]},
          {
           RowBox[{
            RowBox[{"-", 
             FractionBox[
              RowBox[{"3", " ", 
               SuperscriptBox["rv1", "2"], " ", "rv2", " ", 
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", 
                 SqrtBox[
                  RowBox[{
                   SuperscriptBox["rv1", "2"], "+", 
                   SuperscriptBox["rv2", "2"], "+", 
                   SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
              RowBox[{"2", " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}], ")"}], "2"]}]]}], "+", 
            FractionBox[
             RowBox[{"rv2", " ", 
              RowBox[{"Cos", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"2", " ", 
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}]}]], "+", 
            FractionBox[
             RowBox[{"3", " ", 
              SuperscriptBox["rv1", "2"], " ", "rv2", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"rv2", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{
              SuperscriptBox["rv1", "2"], " ", "rv2", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"4", " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]]}]]}]},
          {
           RowBox[{
            RowBox[{"-", 
             FractionBox[
              RowBox[{"3", " ", 
               SuperscriptBox["rv1", "2"], " ", "rv3", " ", 
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", 
                 SqrtBox[
                  RowBox[{
                   SuperscriptBox["rv1", "2"], "+", 
                   SuperscriptBox["rv2", "2"], "+", 
                   SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
              RowBox[{"2", " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}], ")"}], "2"]}]]}], "+", 
            FractionBox[
             RowBox[{"rv3", " ", 
              RowBox[{"Cos", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"2", " ", 
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}]}]], "+", 
            FractionBox[
             RowBox[{"3", " ", 
              SuperscriptBox["rv1", "2"], " ", "rv3", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"rv3", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{
              SuperscriptBox["rv1", "2"], " ", "rv3", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"4", " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]]}]]}]}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {
           RowBox[{
            RowBox[{"-", 
             FractionBox[
              RowBox[{"3", " ", 
               SuperscriptBox["rv1", "2"], " ", "rv2", " ", 
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", 
                 SqrtBox[
                  RowBox[{
                   SuperscriptBox["rv1", "2"], "+", 
                   SuperscriptBox["rv2", "2"], "+", 
                   SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
              RowBox[{"2", " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}], ")"}], "2"]}]]}], "+", 
            FractionBox[
             RowBox[{"rv2", " ", 
              RowBox[{"Cos", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"2", " ", 
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}]}]], "+", 
            FractionBox[
             RowBox[{"3", " ", 
              SuperscriptBox["rv1", "2"], " ", "rv2", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"rv2", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{
              SuperscriptBox["rv1", "2"], " ", "rv2", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"4", " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]]}]]}]},
          {
           RowBox[{
            RowBox[{"-", 
             FractionBox[
              RowBox[{"3", " ", "rv1", " ", 
               SuperscriptBox["rv2", "2"], " ", 
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", 
                 SqrtBox[
                  RowBox[{
                   SuperscriptBox["rv1", "2"], "+", 
                   SuperscriptBox["rv2", "2"], "+", 
                   SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
              RowBox[{"2", " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}], ")"}], "2"]}]]}], "+", 
            FractionBox[
             RowBox[{"rv1", " ", 
              RowBox[{"Cos", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"2", " ", 
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}]}]], "+", 
            FractionBox[
             RowBox[{"3", " ", "rv1", " ", 
              SuperscriptBox["rv2", "2"], " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"rv1", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"rv1", " ", 
              SuperscriptBox["rv2", "2"], " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"4", " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]]}]]}]},
          {
           RowBox[{
            RowBox[{"-", 
             FractionBox[
              RowBox[{"3", " ", "rv1", " ", "rv2", " ", "rv3", " ", 
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", 
                 SqrtBox[
                  RowBox[{
                   SuperscriptBox["rv1", "2"], "+", 
                   SuperscriptBox["rv2", "2"], "+", 
                   SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
              RowBox[{"2", " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}], ")"}], "2"]}]]}], "+", 
            FractionBox[
             RowBox[{"3", " ", "rv1", " ", "rv2", " ", "rv3", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"rv1", " ", "rv2", " ", "rv3", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"4", " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]]}]]}]}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {
           RowBox[{
            RowBox[{"-", 
             FractionBox[
              RowBox[{"3", " ", 
               SuperscriptBox["rv1", "2"], " ", "rv3", " ", 
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", 
                 SqrtBox[
                  RowBox[{
                   SuperscriptBox["rv1", "2"], "+", 
                   SuperscriptBox["rv2", "2"], "+", 
                   SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
              RowBox[{"2", " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}], ")"}], "2"]}]]}], "+", 
            FractionBox[
             RowBox[{"rv3", " ", 
              RowBox[{"Cos", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"2", " ", 
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}]}]], "+", 
            FractionBox[
             RowBox[{"3", " ", 
              SuperscriptBox["rv1", "2"], " ", "rv3", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"rv3", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{
              SuperscriptBox["rv1", "2"], " ", "rv3", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"4", " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]]}]]}]},
          {
           RowBox[{
            RowBox[{"-", 
             FractionBox[
              RowBox[{"3", " ", "rv1", " ", "rv2", " ", "rv3", " ", 
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", 
                 SqrtBox[
                  RowBox[{
                   SuperscriptBox["rv1", "2"], "+", 
                   SuperscriptBox["rv2", "2"], "+", 
                   SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
              RowBox[{"2", " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}], ")"}], "2"]}]]}], "+", 
            FractionBox[
             RowBox[{"3", " ", "rv1", " ", "rv2", " ", "rv3", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"rv1", " ", "rv2", " ", "rv3", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"4", " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]]}]]}]},
          {
           RowBox[{
            RowBox[{"-", 
             FractionBox[
              RowBox[{"3", " ", "rv1", " ", 
               SuperscriptBox["rv3", "2"], " ", 
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", 
                 SqrtBox[
                  RowBox[{
                   SuperscriptBox["rv1", "2"], "+", 
                   SuperscriptBox["rv2", "2"], "+", 
                   SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
              RowBox[{"2", " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}], ")"}], "2"]}]]}], "+", 
            FractionBox[
             RowBox[{"rv1", " ", 
              RowBox[{"Cos", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"2", " ", 
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}]}]], "+", 
            FractionBox[
             RowBox[{"3", " ", "rv1", " ", 
              SuperscriptBox["rv3", "2"], " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"rv1", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"rv1", " ", 
              SuperscriptBox["rv3", "2"], " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"4", " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]]}]]}]}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}]},
     {
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {
           RowBox[{
            RowBox[{"-", 
             FractionBox[
              RowBox[{"3", " ", 
               SuperscriptBox["rv1", "2"], " ", "rv2", " ", 
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", 
                 SqrtBox[
                  RowBox[{
                   SuperscriptBox["rv1", "2"], "+", 
                   SuperscriptBox["rv2", "2"], "+", 
                   SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
              RowBox[{"2", " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}], ")"}], "2"]}]]}], "+", 
            FractionBox[
             RowBox[{"rv2", " ", 
              RowBox[{"Cos", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"2", " ", 
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}]}]], "+", 
            FractionBox[
             RowBox[{"3", " ", 
              SuperscriptBox["rv1", "2"], " ", "rv2", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"rv2", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{
              SuperscriptBox["rv1", "2"], " ", "rv2", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"4", " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]]}]]}]},
          {
           RowBox[{
            RowBox[{"-", 
             FractionBox[
              RowBox[{"3", " ", "rv1", " ", 
               SuperscriptBox["rv2", "2"], " ", 
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", 
                 SqrtBox[
                  RowBox[{
                   SuperscriptBox["rv1", "2"], "+", 
                   SuperscriptBox["rv2", "2"], "+", 
                   SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
              RowBox[{"2", " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}], ")"}], "2"]}]]}], "+", 
            FractionBox[
             RowBox[{"rv1", " ", 
              RowBox[{"Cos", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"2", " ", 
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}]}]], "+", 
            FractionBox[
             RowBox[{"3", " ", "rv1", " ", 
              SuperscriptBox["rv2", "2"], " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"rv1", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"rv1", " ", 
              SuperscriptBox["rv2", "2"], " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"4", " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]]}]]}]},
          {
           RowBox[{
            RowBox[{"-", 
             FractionBox[
              RowBox[{"3", " ", "rv1", " ", "rv2", " ", "rv3", " ", 
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", 
                 SqrtBox[
                  RowBox[{
                   SuperscriptBox["rv1", "2"], "+", 
                   SuperscriptBox["rv2", "2"], "+", 
                   SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
              RowBox[{"2", " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}], ")"}], "2"]}]]}], "+", 
            FractionBox[
             RowBox[{"3", " ", "rv1", " ", "rv2", " ", "rv3", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"rv1", " ", "rv2", " ", "rv3", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"4", " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]]}]]}]}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {
           RowBox[{
            RowBox[{"-", 
             FractionBox[
              RowBox[{"3", " ", "rv1", " ", 
               SuperscriptBox["rv2", "2"], " ", 
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", 
                 SqrtBox[
                  RowBox[{
                   SuperscriptBox["rv1", "2"], "+", 
                   SuperscriptBox["rv2", "2"], "+", 
                   SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
              RowBox[{"2", " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}], ")"}], "2"]}]]}], "+", 
            FractionBox[
             RowBox[{"rv1", " ", 
              RowBox[{"Cos", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"2", " ", 
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}]}]], "+", 
            FractionBox[
             RowBox[{"3", " ", "rv1", " ", 
              SuperscriptBox["rv2", "2"], " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"rv1", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"rv1", " ", 
              SuperscriptBox["rv2", "2"], " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"4", " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]]}]]}]},
          {
           RowBox[{
            RowBox[{"-", 
             FractionBox[
              RowBox[{"3", " ", 
               SuperscriptBox["rv2", "3"], " ", 
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", 
                 SqrtBox[
                  RowBox[{
                   SuperscriptBox["rv1", "2"], "+", 
                   SuperscriptBox["rv2", "2"], "+", 
                   SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
              RowBox[{"2", " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}], ")"}], "2"]}]]}], "+", 
            FractionBox[
             RowBox[{"3", " ", "rv2", " ", 
              RowBox[{"Cos", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"2", " ", 
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}]}]], "+", 
            FractionBox[
             RowBox[{"3", " ", 
              SuperscriptBox["rv2", "3"], " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"3", " ", "rv2", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{
              SuperscriptBox["rv2", "3"], " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"4", " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]]}]]}]},
          {
           RowBox[{
            RowBox[{"-", 
             FractionBox[
              RowBox[{"3", " ", 
               SuperscriptBox["rv2", "2"], " ", "rv3", " ", 
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", 
                 SqrtBox[
                  RowBox[{
                   SuperscriptBox["rv1", "2"], "+", 
                   SuperscriptBox["rv2", "2"], "+", 
                   SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
              RowBox[{"2", " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}], ")"}], "2"]}]]}], "+", 
            FractionBox[
             RowBox[{"rv3", " ", 
              RowBox[{"Cos", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"2", " ", 
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}]}]], "+", 
            FractionBox[
             RowBox[{"3", " ", 
              SuperscriptBox["rv2", "2"], " ", "rv3", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"rv3", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{
              SuperscriptBox["rv2", "2"], " ", "rv3", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"4", " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]]}]]}]}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {
           RowBox[{
            RowBox[{"-", 
             FractionBox[
              RowBox[{"3", " ", "rv1", " ", "rv2", " ", "rv3", " ", 
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", 
                 SqrtBox[
                  RowBox[{
                   SuperscriptBox["rv1", "2"], "+", 
                   SuperscriptBox["rv2", "2"], "+", 
                   SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
              RowBox[{"2", " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}], ")"}], "2"]}]]}], "+", 
            FractionBox[
             RowBox[{"3", " ", "rv1", " ", "rv2", " ", "rv3", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"rv1", " ", "rv2", " ", "rv3", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"4", " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]]}]]}]},
          {
           RowBox[{
            RowBox[{"-", 
             FractionBox[
              RowBox[{"3", " ", 
               SuperscriptBox["rv2", "2"], " ", "rv3", " ", 
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", 
                 SqrtBox[
                  RowBox[{
                   SuperscriptBox["rv1", "2"], "+", 
                   SuperscriptBox["rv2", "2"], "+", 
                   SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
              RowBox[{"2", " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}], ")"}], "2"]}]]}], "+", 
            FractionBox[
             RowBox[{"rv3", " ", 
              RowBox[{"Cos", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"2", " ", 
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}]}]], "+", 
            FractionBox[
             RowBox[{"3", " ", 
              SuperscriptBox["rv2", "2"], " ", "rv3", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"rv3", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{
              SuperscriptBox["rv2", "2"], " ", "rv3", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"4", " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]]}]]}]},
          {
           RowBox[{
            RowBox[{"-", 
             FractionBox[
              RowBox[{"3", " ", "rv2", " ", 
               SuperscriptBox["rv3", "2"], " ", 
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", 
                 SqrtBox[
                  RowBox[{
                   SuperscriptBox["rv1", "2"], "+", 
                   SuperscriptBox["rv2", "2"], "+", 
                   SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
              RowBox[{"2", " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}], ")"}], "2"]}]]}], "+", 
            FractionBox[
             RowBox[{"rv2", " ", 
              RowBox[{"Cos", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"2", " ", 
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}]}]], "+", 
            FractionBox[
             RowBox[{"3", " ", "rv2", " ", 
              SuperscriptBox["rv3", "2"], " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"rv2", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"rv2", " ", 
              SuperscriptBox["rv3", "2"], " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"4", " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]]}]]}]}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}]},
     {
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {
           RowBox[{
            RowBox[{"-", 
             FractionBox[
              RowBox[{"3", " ", 
               SuperscriptBox["rv1", "2"], " ", "rv3", " ", 
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", 
                 SqrtBox[
                  RowBox[{
                   SuperscriptBox["rv1", "2"], "+", 
                   SuperscriptBox["rv2", "2"], "+", 
                   SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
              RowBox[{"2", " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}], ")"}], "2"]}]]}], "+", 
            FractionBox[
             RowBox[{"rv3", " ", 
              RowBox[{"Cos", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"2", " ", 
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}]}]], "+", 
            FractionBox[
             RowBox[{"3", " ", 
              SuperscriptBox["rv1", "2"], " ", "rv3", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"rv3", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{
              SuperscriptBox["rv1", "2"], " ", "rv3", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"4", " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]]}]]}]},
          {
           RowBox[{
            RowBox[{"-", 
             FractionBox[
              RowBox[{"3", " ", "rv1", " ", "rv2", " ", "rv3", " ", 
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", 
                 SqrtBox[
                  RowBox[{
                   SuperscriptBox["rv1", "2"], "+", 
                   SuperscriptBox["rv2", "2"], "+", 
                   SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
              RowBox[{"2", " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}], ")"}], "2"]}]]}], "+", 
            FractionBox[
             RowBox[{"3", " ", "rv1", " ", "rv2", " ", "rv3", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"rv1", " ", "rv2", " ", "rv3", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"4", " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]]}]]}]},
          {
           RowBox[{
            RowBox[{"-", 
             FractionBox[
              RowBox[{"3", " ", "rv1", " ", 
               SuperscriptBox["rv3", "2"], " ", 
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", 
                 SqrtBox[
                  RowBox[{
                   SuperscriptBox["rv1", "2"], "+", 
                   SuperscriptBox["rv2", "2"], "+", 
                   SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
              RowBox[{"2", " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}], ")"}], "2"]}]]}], "+", 
            FractionBox[
             RowBox[{"rv1", " ", 
              RowBox[{"Cos", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"2", " ", 
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}]}]], "+", 
            FractionBox[
             RowBox[{"3", " ", "rv1", " ", 
              SuperscriptBox["rv3", "2"], " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"rv1", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"rv1", " ", 
              SuperscriptBox["rv3", "2"], " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"4", " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]]}]]}]}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {
           RowBox[{
            RowBox[{"-", 
             FractionBox[
              RowBox[{"3", " ", "rv1", " ", "rv2", " ", "rv3", " ", 
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", 
                 SqrtBox[
                  RowBox[{
                   SuperscriptBox["rv1", "2"], "+", 
                   SuperscriptBox["rv2", "2"], "+", 
                   SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
              RowBox[{"2", " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}], ")"}], "2"]}]]}], "+", 
            FractionBox[
             RowBox[{"3", " ", "rv1", " ", "rv2", " ", "rv3", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"rv1", " ", "rv2", " ", "rv3", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"4", " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]]}]]}]},
          {
           RowBox[{
            RowBox[{"-", 
             FractionBox[
              RowBox[{"3", " ", 
               SuperscriptBox["rv2", "2"], " ", "rv3", " ", 
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", 
                 SqrtBox[
                  RowBox[{
                   SuperscriptBox["rv1", "2"], "+", 
                   SuperscriptBox["rv2", "2"], "+", 
                   SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
              RowBox[{"2", " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}], ")"}], "2"]}]]}], "+", 
            FractionBox[
             RowBox[{"rv3", " ", 
              RowBox[{"Cos", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"2", " ", 
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}]}]], "+", 
            FractionBox[
             RowBox[{"3", " ", 
              SuperscriptBox["rv2", "2"], " ", "rv3", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"rv3", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{
              SuperscriptBox["rv2", "2"], " ", "rv3", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"4", " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]]}]]}]},
          {
           RowBox[{
            RowBox[{"-", 
             FractionBox[
              RowBox[{"3", " ", "rv2", " ", 
               SuperscriptBox["rv3", "2"], " ", 
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", 
                 SqrtBox[
                  RowBox[{
                   SuperscriptBox["rv1", "2"], "+", 
                   SuperscriptBox["rv2", "2"], "+", 
                   SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
              RowBox[{"2", " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}], ")"}], "2"]}]]}], "+", 
            FractionBox[
             RowBox[{"rv2", " ", 
              RowBox[{"Cos", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"2", " ", 
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}]}]], "+", 
            FractionBox[
             RowBox[{"3", " ", "rv2", " ", 
              SuperscriptBox["rv3", "2"], " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"rv2", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"rv2", " ", 
              SuperscriptBox["rv3", "2"], " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"4", " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]]}]]}]}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {
           RowBox[{
            RowBox[{"-", 
             FractionBox[
              RowBox[{"3", " ", "rv1", " ", 
               SuperscriptBox["rv3", "2"], " ", 
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", 
                 SqrtBox[
                  RowBox[{
                   SuperscriptBox["rv1", "2"], "+", 
                   SuperscriptBox["rv2", "2"], "+", 
                   SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
              RowBox[{"2", " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}], ")"}], "2"]}]]}], "+", 
            FractionBox[
             RowBox[{"rv1", " ", 
              RowBox[{"Cos", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"2", " ", 
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}]}]], "+", 
            FractionBox[
             RowBox[{"3", " ", "rv1", " ", 
              SuperscriptBox["rv3", "2"], " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"rv1", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"rv1", " ", 
              SuperscriptBox["rv3", "2"], " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"4", " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]]}]]}]},
          {
           RowBox[{
            RowBox[{"-", 
             FractionBox[
              RowBox[{"3", " ", "rv2", " ", 
               SuperscriptBox["rv3", "2"], " ", 
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", 
                 SqrtBox[
                  RowBox[{
                   SuperscriptBox["rv1", "2"], "+", 
                   SuperscriptBox["rv2", "2"], "+", 
                   SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
              RowBox[{"2", " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}], ")"}], "2"]}]]}], "+", 
            FractionBox[
             RowBox[{"rv2", " ", 
              RowBox[{"Cos", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"2", " ", 
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}]}]], "+", 
            FractionBox[
             RowBox[{"3", " ", "rv2", " ", 
              SuperscriptBox["rv3", "2"], " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"rv2", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"rv2", " ", 
              SuperscriptBox["rv3", "2"], " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"4", " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]]}]]}]},
          {
           RowBox[{
            RowBox[{"-", 
             FractionBox[
              RowBox[{"3", " ", 
               SuperscriptBox["rv3", "3"], " ", 
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", 
                 SqrtBox[
                  RowBox[{
                   SuperscriptBox["rv1", "2"], "+", 
                   SuperscriptBox["rv2", "2"], "+", 
                   SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
              RowBox[{"2", " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}], ")"}], "2"]}]]}], "+", 
            FractionBox[
             RowBox[{"3", " ", "rv3", " ", 
              RowBox[{"Cos", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"2", " ", 
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}]}]], "+", 
            FractionBox[
             RowBox[{"3", " ", 
              SuperscriptBox["rv3", "3"], " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"3", " ", "rv3", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{
              SuperscriptBox["rv3", "3"], " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"4", " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]]}]]}]}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}]},
     {
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {
           RowBox[{
            RowBox[{"-", 
             FractionBox[
              RowBox[{
               SuperscriptBox["rv1", "2"], " ", 
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", 
                 SqrtBox[
                  RowBox[{
                   SuperscriptBox["rv1", "2"], "+", 
                   SuperscriptBox["rv2", "2"], "+", 
                   SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
              RowBox[{"4", " ", 
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}]}]]}], "+", 
            FractionBox[
             RowBox[{
              SuperscriptBox["rv1", "2"], " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"2", " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]]}]], "-", 
            FractionBox[
             RowBox[{"Sin", "[", 
              RowBox[{
               FractionBox["1", "2"], " ", 
               SqrtBox[
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}]]}], "]"}], 
             RowBox[{"2", " ", 
              SqrtBox[
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}]]}]]}]},
          {
           RowBox[{
            RowBox[{"-", 
             FractionBox[
              RowBox[{"rv1", " ", "rv2", " ", 
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", 
                 SqrtBox[
                  RowBox[{
                   SuperscriptBox["rv1", "2"], "+", 
                   SuperscriptBox["rv2", "2"], "+", 
                   SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
              RowBox[{"4", " ", 
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}]}]]}], "+", 
            FractionBox[
             RowBox[{"rv1", " ", "rv2", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"2", " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]]}]]}]},
          {
           RowBox[{
            RowBox[{"-", 
             FractionBox[
              RowBox[{"rv1", " ", "rv3", " ", 
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", 
                 SqrtBox[
                  RowBox[{
                   SuperscriptBox["rv1", "2"], "+", 
                   SuperscriptBox["rv2", "2"], "+", 
                   SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
              RowBox[{"4", " ", 
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}]}]]}], "+", 
            FractionBox[
             RowBox[{"rv1", " ", "rv3", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"2", " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]]}]]}]}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {
           RowBox[{
            RowBox[{"-", 
             FractionBox[
              RowBox[{"rv1", " ", "rv2", " ", 
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", 
                 SqrtBox[
                  RowBox[{
                   SuperscriptBox["rv1", "2"], "+", 
                   SuperscriptBox["rv2", "2"], "+", 
                   SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
              RowBox[{"4", " ", 
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}]}]]}], "+", 
            FractionBox[
             RowBox[{"rv1", " ", "rv2", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"2", " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]]}]]}]},
          {
           RowBox[{
            RowBox[{"-", 
             FractionBox[
              RowBox[{
               SuperscriptBox["rv2", "2"], " ", 
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", 
                 SqrtBox[
                  RowBox[{
                   SuperscriptBox["rv1", "2"], "+", 
                   SuperscriptBox["rv2", "2"], "+", 
                   SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
              RowBox[{"4", " ", 
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}]}]]}], "+", 
            FractionBox[
             RowBox[{
              SuperscriptBox["rv2", "2"], " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"2", " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]]}]], "-", 
            FractionBox[
             RowBox[{"Sin", "[", 
              RowBox[{
               FractionBox["1", "2"], " ", 
               SqrtBox[
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}]]}], "]"}], 
             RowBox[{"2", " ", 
              SqrtBox[
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}]]}]]}]},
          {
           RowBox[{
            RowBox[{"-", 
             FractionBox[
              RowBox[{"rv2", " ", "rv3", " ", 
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", 
                 SqrtBox[
                  RowBox[{
                   SuperscriptBox["rv1", "2"], "+", 
                   SuperscriptBox["rv2", "2"], "+", 
                   SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
              RowBox[{"4", " ", 
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}]}]]}], "+", 
            FractionBox[
             RowBox[{"rv2", " ", "rv3", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"2", " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]]}]]}]}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {
           RowBox[{
            RowBox[{"-", 
             FractionBox[
              RowBox[{"rv1", " ", "rv3", " ", 
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", 
                 SqrtBox[
                  RowBox[{
                   SuperscriptBox["rv1", "2"], "+", 
                   SuperscriptBox["rv2", "2"], "+", 
                   SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
              RowBox[{"4", " ", 
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}]}]]}], "+", 
            FractionBox[
             RowBox[{"rv1", " ", "rv3", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"2", " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]]}]]}]},
          {
           RowBox[{
            RowBox[{"-", 
             FractionBox[
              RowBox[{"rv2", " ", "rv3", " ", 
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", 
                 SqrtBox[
                  RowBox[{
                   SuperscriptBox["rv1", "2"], "+", 
                   SuperscriptBox["rv2", "2"], "+", 
                   SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
              RowBox[{"4", " ", 
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}]}]]}], "+", 
            FractionBox[
             RowBox[{"rv2", " ", "rv3", " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"2", " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]]}]]}]},
          {
           RowBox[{
            RowBox[{"-", 
             FractionBox[
              RowBox[{
               SuperscriptBox["rv3", "2"], " ", 
               RowBox[{"Cos", "[", 
                RowBox[{
                 FractionBox["1", "2"], " ", 
                 SqrtBox[
                  RowBox[{
                   SuperscriptBox["rv1", "2"], "+", 
                   SuperscriptBox["rv2", "2"], "+", 
                   SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
              RowBox[{"4", " ", 
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}]}]]}], "+", 
            FractionBox[
             RowBox[{
              SuperscriptBox["rv3", "2"], " ", 
              RowBox[{"Sin", "[", 
               RowBox[{
                FractionBox["1", "2"], " ", 
                SqrtBox[
                 RowBox[{
                  SuperscriptBox["rv1", "2"], "+", 
                  SuperscriptBox["rv2", "2"], "+", 
                  SuperscriptBox["rv3", "2"]}]]}], "]"}]}], 
             RowBox[{"2", " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]]}]], "-", 
            FractionBox[
             RowBox[{"Sin", "[", 
              RowBox[{
               FractionBox["1", "2"], " ", 
               SqrtBox[
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}]]}], "]"}], 
             RowBox[{"2", " ", 
              SqrtBox[
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}]]}]]}]}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{{3.6401134239683447`*^9, 3.640113447151578*^9}, 
   3.640113488479169*^9, {3.640113959108817*^9, 3.6401139691115274`*^9}, {
   3.64011501796024*^9, 3.6401150249632382`*^9}, 3.640115132773779*^9, {
   3.6401152031909747`*^9, 3.6401152593180137`*^9}, 3.6401153060386834`*^9, 
   3.640115692595338*^9, 3.6401523477807093`*^9, 3.640152396911435*^9, 
   3.640152628531938*^9, {3.640152754265111*^9, 3.6401527608361225`*^9}, {
   3.640153029892369*^9, 3.6401530736633053`*^9}, {3.6401531197527447`*^9, 
   3.640153129282771*^9}, {3.640154282100628*^9, 3.6401542850546155`*^9}, 
   3.6401548186003137`*^9, 3.6401551060272145`*^9, 3.640155264523883*^9, 
   3.640155503560177*^9, 3.640155595234197*^9, 3.640155668880672*^9, 
   3.640155769232999*^9, {3.6401558100506067`*^9, 3.6401558265389566`*^9}, {
   3.7203552975112405`*^9, 3.7203553086452403`*^9}, 3.7207914868487544`*^9, 
   3.7207915346127543`*^9, 3.7207915714467545`*^9}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {
           RowBox[{
            FractionBox[
             RowBox[{"3", " ", 
              SuperscriptBox["rv1", "3"]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"3", " ", "rv1"}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           RowBox[{
            FractionBox[
             RowBox[{"3", " ", 
              SuperscriptBox["rv1", "2"], " ", "rv2"}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox["rv2", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           RowBox[{
            FractionBox[
             RowBox[{"3", " ", 
              SuperscriptBox["rv1", "2"], " ", "rv3"}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox["rv3", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {
           RowBox[{
            FractionBox[
             RowBox[{"3", " ", 
              SuperscriptBox["rv1", "2"], " ", "rv2"}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox["rv2", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           RowBox[{
            FractionBox[
             RowBox[{"3", " ", "rv1", " ", 
              SuperscriptBox["rv2", "2"]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox["rv1", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           FractionBox[
            RowBox[{"3", " ", "rv1", " ", "rv2", " ", "rv3"}], 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox["rv1", "2"], "+", 
               SuperscriptBox["rv2", "2"], "+", 
               SuperscriptBox["rv3", "2"]}], ")"}], 
             RowBox[{"5", "/", "2"}]]]}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {
           RowBox[{
            FractionBox[
             RowBox[{"3", " ", 
              SuperscriptBox["rv1", "2"], " ", "rv3"}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox["rv3", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           FractionBox[
            RowBox[{"3", " ", "rv1", " ", "rv2", " ", "rv3"}], 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox["rv1", "2"], "+", 
               SuperscriptBox["rv2", "2"], "+", 
               SuperscriptBox["rv3", "2"]}], ")"}], 
             RowBox[{"5", "/", "2"}]]]},
          {
           RowBox[{
            FractionBox[
             RowBox[{"3", " ", "rv1", " ", 
              SuperscriptBox["rv3", "2"]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox["rv1", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}]},
     {
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {
           RowBox[{
            FractionBox[
             RowBox[{"3", " ", 
              SuperscriptBox["rv1", "2"], " ", "rv2"}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox["rv2", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           RowBox[{
            FractionBox[
             RowBox[{"3", " ", "rv1", " ", 
              SuperscriptBox["rv2", "2"]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox["rv1", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           FractionBox[
            RowBox[{"3", " ", "rv1", " ", "rv2", " ", "rv3"}], 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox["rv1", "2"], "+", 
               SuperscriptBox["rv2", "2"], "+", 
               SuperscriptBox["rv3", "2"]}], ")"}], 
             RowBox[{"5", "/", "2"}]]]}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {
           RowBox[{
            FractionBox[
             RowBox[{"3", " ", "rv1", " ", 
              SuperscriptBox["rv2", "2"]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox["rv1", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           RowBox[{
            FractionBox[
             RowBox[{"3", " ", 
              SuperscriptBox["rv2", "3"]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"3", " ", "rv2"}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           RowBox[{
            FractionBox[
             RowBox[{"3", " ", 
              SuperscriptBox["rv2", "2"], " ", "rv3"}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox["rv3", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {
           FractionBox[
            RowBox[{"3", " ", "rv1", " ", "rv2", " ", "rv3"}], 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox["rv1", "2"], "+", 
               SuperscriptBox["rv2", "2"], "+", 
               SuperscriptBox["rv3", "2"]}], ")"}], 
             RowBox[{"5", "/", "2"}]]]},
          {
           RowBox[{
            FractionBox[
             RowBox[{"3", " ", 
              SuperscriptBox["rv2", "2"], " ", "rv3"}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox["rv3", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           RowBox[{
            FractionBox[
             RowBox[{"3", " ", "rv2", " ", 
              SuperscriptBox["rv3", "2"]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox["rv2", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}]},
     {
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {
           RowBox[{
            FractionBox[
             RowBox[{"3", " ", 
              SuperscriptBox["rv1", "2"], " ", "rv3"}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox["rv3", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           FractionBox[
            RowBox[{"3", " ", "rv1", " ", "rv2", " ", "rv3"}], 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox["rv1", "2"], "+", 
               SuperscriptBox["rv2", "2"], "+", 
               SuperscriptBox["rv3", "2"]}], ")"}], 
             RowBox[{"5", "/", "2"}]]]},
          {
           RowBox[{
            FractionBox[
             RowBox[{"3", " ", "rv1", " ", 
              SuperscriptBox["rv3", "2"]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox["rv1", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {
           FractionBox[
            RowBox[{"3", " ", "rv1", " ", "rv2", " ", "rv3"}], 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox["rv1", "2"], "+", 
               SuperscriptBox["rv2", "2"], "+", 
               SuperscriptBox["rv3", "2"]}], ")"}], 
             RowBox[{"5", "/", "2"}]]]},
          {
           RowBox[{
            FractionBox[
             RowBox[{"3", " ", 
              SuperscriptBox["rv2", "2"], " ", "rv3"}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox["rv3", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           RowBox[{
            FractionBox[
             RowBox[{"3", " ", "rv2", " ", 
              SuperscriptBox["rv3", "2"]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox["rv2", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {
           RowBox[{
            FractionBox[
             RowBox[{"3", " ", "rv1", " ", 
              SuperscriptBox["rv3", "2"]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox["rv1", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           RowBox[{
            FractionBox[
             RowBox[{"3", " ", "rv2", " ", 
              SuperscriptBox["rv3", "2"]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox["rv2", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           RowBox[{
            FractionBox[
             RowBox[{"3", " ", 
              SuperscriptBox["rv3", "3"]}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"5", "/", "2"}]]], "-", 
            FractionBox[
             RowBox[{"3", " ", "rv3"}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}]},
     {
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {
           RowBox[{
            RowBox[{"-", 
             FractionBox[
              SuperscriptBox["rv1", "2"], 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]]]}], "+", 
            FractionBox["1", 
             SqrtBox[
              RowBox[{
               SuperscriptBox["rv1", "2"], "+", 
               SuperscriptBox["rv2", "2"], "+", 
               SuperscriptBox["rv3", "2"]}]]]}]},
          {
           RowBox[{"-", 
            FractionBox[
             RowBox[{"rv1", " ", "rv2"}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           RowBox[{"-", 
            FractionBox[
             RowBox[{"rv1", " ", "rv3"}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {
           RowBox[{"-", 
            FractionBox[
             RowBox[{"rv1", " ", "rv2"}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           RowBox[{
            RowBox[{"-", 
             FractionBox[
              SuperscriptBox["rv2", "2"], 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]]]}], "+", 
            FractionBox["1", 
             SqrtBox[
              RowBox[{
               SuperscriptBox["rv1", "2"], "+", 
               SuperscriptBox["rv2", "2"], "+", 
               SuperscriptBox["rv3", "2"]}]]]}]},
          {
           RowBox[{"-", 
            FractionBox[
             RowBox[{"rv2", " ", "rv3"}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {
           RowBox[{"-", 
            FractionBox[
             RowBox[{"rv1", " ", "rv3"}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           RowBox[{"-", 
            FractionBox[
             RowBox[{"rv2", " ", "rv3"}], 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
                SuperscriptBox["rv1", "2"], "+", 
                SuperscriptBox["rv2", "2"], "+", 
                SuperscriptBox["rv3", "2"]}], ")"}], 
              RowBox[{"3", "/", "2"}]]]}]},
          {
           RowBox[{
            RowBox[{"-", 
             FractionBox[
              SuperscriptBox["rv3", "2"], 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{
                 SuperscriptBox["rv1", "2"], "+", 
                 SuperscriptBox["rv2", "2"], "+", 
                 SuperscriptBox["rv3", "2"]}], ")"}], 
               RowBox[{"3", "/", "2"}]]]}], "+", 
            FractionBox["1", 
             SqrtBox[
              RowBox[{
               SuperscriptBox["rv1", "2"], "+", 
               SuperscriptBox["rv2", "2"], "+", 
               SuperscriptBox["rv3", "2"]}]]]}]}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{{3.6401134239683447`*^9, 3.640113447151578*^9}, 
   3.640113488479169*^9, {3.640113959108817*^9, 3.6401139691115274`*^9}, {
   3.64011501796024*^9, 3.6401150249632382`*^9}, 3.640115132773779*^9, {
   3.6401152031909747`*^9, 3.6401152593180137`*^9}, 3.6401153060386834`*^9, 
   3.640115692595338*^9, 3.6401523477807093`*^9, 3.640152396911435*^9, 
   3.640152628531938*^9, {3.640152754265111*^9, 3.6401527608361225`*^9}, {
   3.640153029892369*^9, 3.6401530736633053`*^9}, {3.6401531197527447`*^9, 
   3.640153129282771*^9}, {3.640154282100628*^9, 3.6401542850546155`*^9}, 
   3.6401548186003137`*^9, 3.6401551060272145`*^9, 3.640155264523883*^9, 
   3.640155503560177*^9, 3.640155595234197*^9, 3.640155668880672*^9, 
   3.640155769232999*^9, {3.6401558100506067`*^9, 3.6401558265389566`*^9}, {
   3.7203552975112405`*^9, 3.7203553086452403`*^9}, 3.7207914868487544`*^9, 
   3.7207915346127543`*^9, 3.720791571674754*^9}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {"0"},
          {"0"},
          {"0"},
          {
           RowBox[{
            FractionBox["1", "2"], " ", 
            RowBox[{"Cos", "[", 
             FractionBox["aa4", "2"], "]"}]}]}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {"0"},
          {"0"},
          {"0"},
          {"0"}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {"0"},
          {"0"},
          {"0"},
          {"0"}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {
           RowBox[{
            FractionBox["1", "2"], " ", 
            RowBox[{"Cos", "[", 
             FractionBox["aa4", "2"], "]"}]}]},
          {"0"},
          {"0"},
          {
           RowBox[{
            RowBox[{"-", 
             FractionBox["1", "4"]}], " ", "aa1", " ", 
            RowBox[{"Sin", "[", 
             FractionBox["aa4", "2"], "]"}]}]}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}]},
     {
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {"0"},
          {"0"},
          {"0"},
          {"0"}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {"0"},
          {"0"},
          {"0"},
          {
           RowBox[{
            FractionBox["1", "2"], " ", 
            RowBox[{"Cos", "[", 
             FractionBox["aa4", "2"], "]"}]}]}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {"0"},
          {"0"},
          {"0"},
          {"0"}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {"0"},
          {
           RowBox[{
            FractionBox["1", "2"], " ", 
            RowBox[{"Cos", "[", 
             FractionBox["aa4", "2"], "]"}]}]},
          {"0"},
          {
           RowBox[{
            RowBox[{"-", 
             FractionBox["1", "4"]}], " ", "aa2", " ", 
            RowBox[{"Sin", "[", 
             FractionBox["aa4", "2"], "]"}]}]}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}]},
     {
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {"0"},
          {"0"},
          {"0"},
          {"0"}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {"0"},
          {"0"},
          {"0"},
          {"0"}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {"0"},
          {"0"},
          {"0"},
          {
           RowBox[{
            FractionBox["1", "2"], " ", 
            RowBox[{"Cos", "[", 
             FractionBox["aa4", "2"], "]"}]}]}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {"0"},
          {"0"},
          {
           RowBox[{
            FractionBox["1", "2"], " ", 
            RowBox[{"Cos", "[", 
             FractionBox["aa4", "2"], "]"}]}]},
          {
           RowBox[{
            RowBox[{"-", 
             FractionBox["1", "4"]}], " ", "aa3", " ", 
            RowBox[{"Sin", "[", 
             FractionBox["aa4", "2"], "]"}]}]}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}]},
     {
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {"0"},
          {"0"},
          {"0"},
          {"0"}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {"0"},
          {"0"},
          {"0"},
          {"0"}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {"0"},
          {"0"},
          {"0"},
          {"0"}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {"0"},
          {"0"},
          {"0"},
          {
           RowBox[{
            RowBox[{"-", 
             FractionBox["1", "4"]}], " ", 
            RowBox[{"Cos", "[", 
             FractionBox["aa4", "2"], "]"}]}]}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{{3.6401134239683447`*^9, 3.640113447151578*^9}, 
   3.640113488479169*^9, {3.640113959108817*^9, 3.6401139691115274`*^9}, {
   3.64011501796024*^9, 3.6401150249632382`*^9}, 3.640115132773779*^9, {
   3.6401152031909747`*^9, 3.6401152593180137`*^9}, 3.6401153060386834`*^9, 
   3.640115692595338*^9, 3.6401523477807093`*^9, 3.640152396911435*^9, 
   3.640152628531938*^9, {3.640152754265111*^9, 3.6401527608361225`*^9}, {
   3.640153029892369*^9, 3.6401530736633053`*^9}, {3.6401531197527447`*^9, 
   3.640153129282771*^9}, {3.640154282100628*^9, 3.6401542850546155`*^9}, 
   3.6401548186003137`*^9, 3.6401551060272145`*^9, 3.640155264523883*^9, 
   3.640155503560177*^9, 3.640155595234197*^9, 3.640155668880672*^9, 
   3.640155769232999*^9, {3.6401558100506067`*^9, 3.6401558265389566`*^9}, {
   3.7203552975112405`*^9, 3.7203553086452403`*^9}, 3.7207914868487544`*^9, 
   3.7207915346127543`*^9, 3.7207915717057543`*^9}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {"0"},
          {"0"},
          {"0"},
          {"1"}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {"0"},
          {"0"},
          {"0"},
          {"0"}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {"0"},
          {"0"},
          {"0"},
          {"0"}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {"1"},
          {"0"},
          {"0"},
          {"0"}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}]},
     {
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {"0"},
          {"0"},
          {"0"},
          {"0"}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {"0"},
          {"0"},
          {"0"},
          {"1"}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {"0"},
          {"0"},
          {"0"},
          {"0"}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {"0"},
          {"1"},
          {"0"},
          {"0"}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}]},
     {
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {"0"},
          {"0"},
          {"0"},
          {"0"}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {"0"},
          {"0"},
          {"0"},
          {"0"}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {"0"},
          {"0"},
          {"0"},
          {"1"}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {"0"},
          {"0"},
          {"1"},
          {"0"}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}],
        Column], "\[NoBreak]", ")"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{{3.6401134239683447`*^9, 3.640113447151578*^9}, 
   3.640113488479169*^9, {3.640113959108817*^9, 3.6401139691115274`*^9}, {
   3.64011501796024*^9, 3.6401150249632382`*^9}, 3.640115132773779*^9, {
   3.6401152031909747`*^9, 3.6401152593180137`*^9}, 3.6401153060386834`*^9, 
   3.640115692595338*^9, 3.6401523477807093`*^9, 3.640152396911435*^9, 
   3.640152628531938*^9, {3.640152754265111*^9, 3.6401527608361225`*^9}, {
   3.640153029892369*^9, 3.6401530736633053`*^9}, {3.6401531197527447`*^9, 
   3.640153129282771*^9}, {3.640154282100628*^9, 3.6401542850546155`*^9}, 
   3.6401548186003137`*^9, 3.6401551060272145`*^9, 3.640155264523883*^9, 
   3.640155503560177*^9, 3.640155595234197*^9, 3.640155668880672*^9, 
   3.640155769232999*^9, {3.6401558100506067`*^9, 3.6401558265389566`*^9}, {
   3.7203552975112405`*^9, 3.7203553086452403`*^9}, 3.7207914868487544`*^9, 
   3.7207915346127543`*^9, 3.7207915717107544`*^9}]
}, Open  ]]
}, Open  ]]
},
AutoGeneratedPackage->None,
WindowSize->{1904, 943},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
ShowSelection->True,
CellLabelAutoDelete->True,
FrontEndVersion->"11.0 for Microsoft Windows (64-bit) (September 21, 2016)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> 
     FrontEnd`FileName[{"Report"}, "StandardReport.nb", CharacterEncoding -> 
       "UTF-8"]], Background -> GrayLevel[1]], 
   Cell[
    StyleData["Title"], TextAlignment -> Center, FontColor -> GrayLevel[0]], 
   Cell[
    StyleData["Section"], CellFrameColor -> GrayLevel[0.334844]], 
   Cell[
    StyleData["Input"]]}, Visible -> False, FrontEndVersion -> 
  "11.0 for Microsoft Windows (64-bit) (September 21, 2016)", 
  StyleDefinitions -> "PrivateStylesheetFormatting.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[567, 22, 223, 3, 35, "Subsubsection"],
Cell[CellGroupData[{
Cell[815, 29, 35153, 894, 2586, "Input"],
Cell[35971, 925, 42163, 1117, 539, "Output"],
Cell[78137, 2044, 29169, 785, 497, "Output"],
Cell[107309, 2831, 110813, 2825, 539, "Output"],
Cell[218125, 5658, 30414, 803, 451, "Output"],
Cell[248542, 6463, 13282, 364, 357, "Output"],
Cell[261827, 6829, 9193, 247, 239, "Output"]
}, Open  ]]
}, Open  ]]
}
]
*)

